/* File name     : cc_common.h
 * Creation date : July 1992
 * Modified date : Feburary 1993
 * Author        : Y.Yasu, Online group, Physics department, KEK
 */

/* CAMAC status */
#define CC_STA_OK               0
#define CC_STA_SINGLE_TIMEOUT   -1
#define CC_STA_BLOCK_INVFUNC    -2
#define CC_STA_BLOCK_TIMEOUT    -3
#define CC_STA_LIST_STOP        -10
#define CC_STA_LIST_TIMEOUT     -11
#define CC_STA_LIST_EMPTY       -12
#define CC_STA_LIST_ZERODIV     -13
#define CC_STA_LIST_OVERSTEP    -14
#define CC_STA_KLIST_SAMENAME   -21
#define CC_STA_KLIST_MEMORYFULL -22
#define CC_STA_KLIST_NONAME     -23

/* ioctl command */
#define CCIOC_CHK_BRANCH        _IOR('c',0,int) /* check branch # */
#define CCIOC_SET_BRANCH        _IOW('c',1,int) /* set branch # */
#define CCIOC_SET_CRATE         _IOW('c',2,int) /* set crate # */
#define CCIOC_WAIT_LAM          _IOW('c',3,int) /* wait LAM interrupt */
#define CCIOC_ENABLE_LAM        _IOW('c',4,int) /* enable LAM interrupt */
#define CCIOC_DISABLE_LAM       _IOW('c',5,int) /* disable LAM interrupt */
#define CCIOC_REG_DUMP          _IOR('c',6,struct reg2917s)
#define CCIOC_GENZ              _IOW('c',7,int)
#define CCIOC_GENC              _IOW('c',8,int)
#define CCIOC_REMI              _IOW('c',9,int)
#define CCIOC_SETI              _IOW('c',10,int)
#define CCIOC_RESET             _IOW('c',11,int)
#define CCIOC_CMD_DUMP          _IOWR('c',12,struct camcmd)
#define CCIOC_WAIT_SIG          _IOW('c',13,int)
#define CCIOC_SET_DEBUG         _IOW('c',14,int)
#define CCIOC_READ_STATUS       _IOR('c',15,int)
#define CCIOC_KINIT             _IOW('c',16,int)

/* command for message structure */
#define CC_CMD_DOSINGLE         1
#define CC_CMD_DOBLOCK          2
#define CC_CMD_LOADLIST         3
#define CC_CMD_LOADDOLIST       4
#define CC_CMD_DOLIST           5
#define CC_CMD_ADDKLIST         6
#define CC_CMD_DELKLIST         7
#define CC_CMD_EXEKLIST         8
#define CC_CMD_SAVEKLIST        9
#define CC_CMD_LOADKLIST       10

#define CC_K_MAX_BRANCH 1
#define CC_K_MAX_CRATE  8

/* signal */
#define CC_K_SIGNAL             SIGUSR1

/* for Kinetic List Processing */
#define CC_KLIST_LENMEM         8000
#define CC_KLIST_LENNAM         9

/* NAF code generation */
#define NAFGEN(n,a,f) ((((n<<4) + a) << 5) + f)

/*
 Description of status
   There are two types of status, system status and camac status.
   The system status means return camlib function value while the camac status
   means the return value of status of iosb. For examples, CC_STA_OK,
   CC_STA_SINGLE_TIMEOUT,... are the camac status.  And, ENXIO, EBUSY,.. are
   the system status which is defined in the files of the directory
   /usr/include.
   Therefore,
   system status = 0 and camac status = 0 : all are ok.
   system status = -1 and camac status !=0 : system call is ok, but camac fails
       : or list operation fails.
   system status !=0 except -1            : system call is bad.
 * status :
   camac status  : success = 0 while error != 0. The error is negative value
   which is defined in this file.
 * ret_length :
   actual length of data transferred in short word
 * s_register :
   Only a parameter which is passed to device driver.
   latest contents of s register is return
 * devinfo
    first short word  : Control Status Register(0:15)
    second short word : Channel Status/Error Register(0:4)
*/
struct cc_iosb {
    int status;
    int ret_length;
    int s_reg;
    int devinfo;
};

struct message {
    unsigned short command;
    unsigned short mode;
    unsigned short naf;
    unsigned short *ptr_data;
    unsigned short *ptr_qx;
    struct cc_iosb *ptr_iosb;
    char *klname;
};

struct camcmd {
    short cmdadr;
    short cmd;
    short naf;
    short wc;
    short dummy;
    short halt;
};

struct reg2917s {
    u_short cser;
    u_short docr;
    u_short sccr;
    u_short mtcr;
    u_short machi;
    u_short maclo;
    u_short lamicr;
    u_short donicr;
    u_short empicr;
    u_short aboicr;
    u_short lamivr;
    u_short donivr;
    u_short empivr;
    u_short aboivr;
    u_short cmar;
    u_short cwcr;
    u_short srr;
    u_short csr;
};
