/* File name     : camlib.c
 * Creation date : August 1991
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Modified      : August 1992, Y.Y
 *               :     Add list processing.
 *               :     Kinetic List Processing is based on the code for
 *               :     SunSparc by Y.Takeuchi(TIT).
 */

#include <sys/types.h>
#include <sys/uio.h>
#include <sys/file.h>
#include <stdio.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <sys/signal.h>
#include "camlib.h"

/* Local Storage */
static int file_pointer;
static struct iovec vec[3];
static struct cc_iosb iosb;
static struct message message;
static int cc_path;
static u_short klist[CC_KLIST_LENMEM];

/* CAM_Open, COPEN, CAMOPN */
int cam_open_( ) { return( COPEN( )); }
int camopn_( ) { return( COPEN( )); }
int copen_( ) { return( COPEN( )); }
int COPEN( ) {

    cc_path = open("/dev/cc0", O_RDWR);
    if( cc_path == -1 ) {
	    return ENODEV;
    }
    file_pointer = 0;
    return 0;
}

/* CAM_Close, CAMCLS, CCLOSE */
int cam_close_( ) { return( CCLOSE( ) ); }
int camcls_( ) { return( CCLOSE( ) ); }
int cclose_( ) { return( CCLOSE( ) ); }
int CCLOSE( ) {

    if( (close(cc_path)) == -1 )
        return errno;
    return 0;
}

/* NAF */
int naf_( n, a, f )
int *n, *a, *f; {
    return (NAF( *n, *a, *f ));
}
int NAF( n, a, f )
int n, a, f; {
    return  ((((n<<4) + a) << 5) + f);
}

/* CRESET */
int creset_() { return( CRESET() ); }
int CRESET() {
    int i;

    if( ioctl(cc_path, CCIOC_RESET, &i) )
        return errno;
    return 0;
}

/* CSETBR */
int csetbr_( branch ) int *branch; { return( CSETBR(*branch) ); }
int CSETBR( branch )
int branch; {
    int status, max_branch;

    if( (branch > (CC_K_MAX_BRANCH-1)) || (branch < 0) )
	return EINVAL;
    if( ioctl(cc_path, CCIOC_SET_BRANCH, &branch) )
        return errno;
    return 0;
}

/* CSETCR */
int csetcr_( crate ) int *crate; { return( CSETCR(*crate) ); }
int CSETCR( crate )
int crate; {
    int status;

    if( (crate > (CC_K_MAX_CRATE-1)) || (crate < 0) )
	return EINVAL;
    if( ioctl(cc_path, CCIOC_SET_CRATE, &crate ) )
        return errno;
    return 0;
}

int cdmprg_(regs) struct reg2917s *regs; { return ( CDMPRG(regs) ); }
int CDMPRG( regs )
struct reg2917s *regs; {

    if( ioctl(cc_path, CCIOC_REG_DUMP, regs ) )
        return errno;
    return 0;
}

int cdmpcm_(cmds) struct camcmd *cmds; { return ( CDMPRG(cmds) ); }
int CDMPCM( cmds )
struct camcmd *cmds; {

    if( ioctl(cc_path, CCIOC_CMD_DUMP, cmds ) )
        return errno;
    return 0;
}

/* CGENZ */
int cgenz_() { return( CGENZ() ); }
int CGENZ() {
    int status, q, x = 0;
    short i=0;

    status = CAMACW(NAFGEN(30,0,1),&i,&q,&x);
    if( status != 0 )
	return status;
    i |= 1;
    status = CAMACW(NAFGEN(30,0,17),&i,&q,&x);
    if( status != 0 )
	return status;
    return status;
}

/* CGENC */
int cgenc_() { return( CGENC() ); }
int CGENC() {
    int status, q, x;
    short i=0;

    status = CAMACW(NAFGEN(30,0,1),&i,&q,&x);
    if( status != 0 )
	return status;
    i |= 2;
    return CAMACW(NAFGEN(30,0,17),&i,&q,&x);
}

/* CSETI */
int cseti_() { return( CSETI() ); }
int CSETI() {
    int status, q, x;
    short i=0;

    status = CAMACW(NAFGEN(30,0,1),&i,&q,&x);
    if( status != 0 )
	return status;
    i |= 4;
    return CAMACW(NAFGEN(30,0,17),&i,&q,&x);
}

/* CREMI */
int cremi_() { return( CREMI() ); }
int CREMI() {
    int status, q, x;
    short i=0;

    status = CAMACW(NAFGEN(30,0,1),&i,&q,&x);
    if( status != 0 )
	return status;
    i &= ~4;
    return CAMACW(NAFGEN(30,0,17),&i,&q,&x);
}

/* CAMACW */
int camacw_( naf, data, q, x )
u_short *data;
int *naf, *q, *x; {
    return CAMACW( *naf, data, q, x );
}
int CAMACW(naf, data, q, x)
u_short *data;
int naf, *q, *x; {
    int status;
    int qx;

    message.command = (unsigned short)CC_CMD_DOSINGLE;
    message.mode = (unsigned short)CC_K_MODE_WS16;
    message.naf = (unsigned short)naf;
    message.ptr_data = data;
    message.ptr_iosb = &iosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    errno = 0;
    if( (writev(cc_path, vec, 1) ) < 0 )
	return errno;
    qx = ~(iosb.devinfo>>1);
    *q = qx & 1;
    *x = (qx>>1)&1;
    check_file_pointer(4);
    return errno;
}

/* CAMACL, CAMAC */
int camacl_( naf, data, q, x )
int *naf, *data, *q, *x; {
    return CAMAC(*naf, data, q, x);
}
int camac_( naf, data, q, x )
int *naf, *data, *q, *x; {
    return CAMAC(*naf, data, q, x);
}
int CAMAC(naf, data, q, x)
int naf, *data, *q, *x; {
    int status;
    int qx;

    message.command = (unsigned short)CC_CMD_DOSINGLE;
    message.mode = (unsigned short)CC_K_MODE_WS24;
    message.naf = (unsigned short)naf;
    message.ptr_data = (unsigned short *)data;
    message.ptr_iosb = &iosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    errno = 0;
    if( (writev(cc_path, vec, 1) ) < 0 )
	return errno;
    qx = ~(iosb.devinfo>>1);
    *q = qx & 1;
    *x = (qx>>1)&1;
    check_file_pointer(4);
    return errno;
}

/* Cam_EnableLAM, CENLAM, CELAM */
int cam_enablelam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int cenlam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int celam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int CENLAM(mask)
int mask; {
    int status;

    errno = 0;
    if (ioctl(cc_path, CCIOC_ENABLE_LAM, &mask) == -1 )
        return errno;
    return 0;
}

/* Cam_DisableLAM, CDSLAM, CDLAM */
int cam_disablelam_() {
    return CDSLAM();
}
int cdslam_() {
    return CDSLAM();
}
int cdlam_() {
    return CDSLAM();
}
int CDSLAM() {
    int mask = 0;
    int status;

    errno = 0;
    if (ioctl(cc_path, CCIOC_DISABLE_LAM, &mask) == -1 )
        return errno;
    return 0;
}

/* Cam_WaitLAM, CWTLAM, CWLAM */
cam_waitlam_( timeout )
int *timeout; {
    return CWTLAM( *timeout );
}
int cwtlam_( timeout )
int *timeout; {
    return CWTLAM( *timeout );
}
int cwlam_( timeout )
int *timeout; {
    return CWTLAM( *timeout );
}
int CWTLAM(timeout)
int timeout; {
    int status;

    errno = 0;
    if (ioctl(cc_path, CCIOC_WAIT_LAM, &timeout) == -1 )
        return errno;
    return 0;
}

/* CESIG : not supported */
cesig_() { CESIG(); }
CESIG(funct)
void (*funct)();
{
    errno = 0;
    signal(CC_K_SIGNAL, funct);
    return 0;
}

/* CDSIG : not supported */
cdsig_() { CDSIG(); }
CDSIG()
{
    signal(CC_K_SIGNAL, SIG_DFL);
    return 0;
}

/* CWSIG : not supported */
cwsig_() { CWSIG(); }
CWSIG()
{
    int pid, pgid;

    errno = 0;
    pgid = pid = getpid();
    if( setpgid(pid, pgid) == -1 )
        return errno;
    if( ioctl(cc_path, CCIOC_WAIT_SIG, &pgid) == -1 )
        return errno;
    return 0;
}

/* CBWWAI, CDMAW, CamBloWorWAIt */
int cambloworwait_( mode, naf, buffer, length, ret_len, err_code )
u_short *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS16,
	 *naf, buffer, (*length)*sizeof(short), ret_len, err_code );
}
int cbwwai_( mode, naf, buffer, length, ret_len, err_code )
u_short *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS16,
	 *naf, buffer, (*length)*sizeof(short), ret_len, err_code );
}
int cdmaw_( mode, naf, buffer, length, ret_len, err_code )
u_short *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
         (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS16,
	 *naf, buffer, (*length)*sizeof(short), ret_len, err_code );
}
int CDMAW(mode, naf, buffer, length, ret_len, err_code )
u_short *buffer;
int mode, naf, length, *ret_len, *err_code; {
    return CamBlokWait_common(
         (mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS16,
	 naf, buffer, length*sizeof(short), ret_len, err_code );
}

/* CBLWAI, CDMAL, CamBloLonWAIt */
int camblolonwait_( mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS24,
	 *naf, buffer, (*length)*sizeof(long), *ret_len, err_code );
}
int cblwai_( mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS24,
	 *naf, buffer, (*length)*sizeof(long), *ret_len, err_code );
}
int cdmal_(mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
         (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS24,
	 *naf, buffer, (*length)*sizeof(long), ret_len, err_code );
}
int CDMAL(mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int mode, naf, length, *ret_len, *err_code; {
    return CamBlokWait_common(
         (mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS24,
	 naf, buffer, length*sizeof(long), ret_len, err_code );
}

int CamBlokWait_common(mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int mode, naf, length, *ret_len, *err_code; {
    int status;

    message.command = (u_short)CC_CMD_DOBLOCK;
    message.mode = (u_short)mode;
    message.naf = (u_short)naf;
    message.ptr_iosb = &iosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    vec[1].iov_base = (caddr_t)buffer;
    vec[1].iov_len = length;
    errno = 0;
    if( (readv(cc_path, vec, 2)) < 0 )
	return errno;
    *ret_len = iosb.ret_length;
    *err_code = iosb.status;
    check_file_pointer(vec[0].iov_len + vec[1].iov_len);
    return 0;
}

/* ==================== FOR LIST PROCESSING ====================== */

/* CamLisExeWAIt, CLEWAI */
camlisexewait_(list, lenlis, buf, lenbuf, iosb)
u_short *list, *buf;
int *lenlis, *lenbuf;
struct cc_iosb *iosb;
{
    return CLEWAI(list, *lenlis, buf, *lenbuf, iosb);
}
clewai_(list, lenlis, buf, lenbuf, iosb)
u_short *list, *buf;
int *lenlis, *lenbuf;
struct cc_iosb *iosb;
{
    return CLEWAI(list, *lenlis, buf, *lenbuf, iosb);
}
CLEWAI(list, lenlis, buf, lenbuf, iiosb)
u_short *list, *buf;
int lenlis, lenbuf;
struct cc_iosb *iiosb;
{
    message.command = (u_short)CC_CMD_LOADDOLIST;
    message.ptr_iosb = (struct cc_iosb *)iiosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    vec[1].iov_base = (caddr_t)list;
    vec[1].iov_len = lenlis * sizeof(u_short);
    vec[2].iov_base = (caddr_t)buf;
    vec[2].iov_len = lenbuf * sizeof(u_short);
    errno = 0;
    if( (readv(cc_path, vec, 3)) < 0 )
	return errno;
    check_file_pointer(vec[0].iov_len+vec[1].iov_len+vec[2].iov_len);
    return 0;
}

/* CamExeWAIt, CEWAI */
camexewait_(buf, lenbuf, iosb)
u_short *buf;
int *lenbuf;
struct cc_iosb *iosb;
{
    return CEWAI(buf, *lenbuf, iosb);
}

cewai_(buf, lenbuf, iosb)
u_short *buf;
int *lenbuf;
struct cc_iosb *iosb;
{
    return CEWAI(buf, *lenbuf, iosb);
}

CEWAI(buf, lenbuf, iiosb)
u_short *buf;
int lenbuf;
struct cc_iosb *iiosb;
{
    message.command = (u_short)CC_CMD_DOLIST;
    message.ptr_iosb = (struct cc_iosb *)iiosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    vec[1].iov_base = (caddr_t)buf;
    vec[1].iov_len = lenbuf * sizeof(u_short);
    errno = 0;
    if( (readv(cc_path, vec, 2)) < 0 )
	return errno;
    check_file_pointer(vec[0].iov_len+vec[1].iov_len);
    return 0;
}

/* CamLisLOAD, CLLOAD */
camlisload_(list, lenlis, err)
u_short *list;
int *lenlis;
int *err;
{
    return CLLOAD(list, *lenlis, err);
}
clload_(list, lenlis, err)
u_short *list;
int *lenlis;
int *err;
{
    return CLLOAD(list, *lenlis, err);
}
CLLOAD(list, lenlis, err)
u_short *list;
int lenlis;
int *err;
{
    *err = 0;
    message.command = (u_short)CC_CMD_LOADLIST;
    message.ptr_iosb = (struct cc_iosb *)&iosb;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    vec[1].iov_base = (caddr_t)list;
    vec[1].iov_len = lenlis * sizeof(u_short);
    errno = 0;
    if( (readv(cc_path, vec, 2)) < 0 )
	return errno;
    check_file_pointer(vec[0].iov_len+vec[1].iov_len);
    return 0;
}

/* CamReaLIST, CRLIST */
camrealist_(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int *lenlis, *retlen, *err;
{
    return CRLIST(fname, list, *lenlis, retlen, err);
}
crlist_(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int *lenlis, *retlen, *err;
{
    return CRLIST(fname, list, *lenlis, retlen, err);
}
CRLIST(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int lenlis, *retlen, *err;
{
    register FILE *fp;
    int size, a, b, c, d;

    if ((fp = fopen(fname, "r")) == NULL) {
        *err = errno;
        return errno;
    }
    (void)fscanf(fp, "%X", &size);
    if (size > lenlis) {
        *err = -1;
        return -1;
    }
    *retlen = size;
    while (size > 0) {
        (void)fscanf(fp, "%X%X%X%X", &a, &b, &c, &d);
        *list++ = a;
        *list++ = b;
        *list++ = c;
        *list++ = d;
        size -= 4;
    }
    *err = 0;
    return fclose(fp);
}
    
/* ======================== Kinetic List Processing =================== */


/* CKINIT */
int ckinit_( ) { return( CKINIT( )); }
int CKINIT()
{
    int dummy;
    errno = 0;
    ioctl(cc_path, CCIOC_KINIT, &dummy);
    return errno;
}

/* CKADD */
int ckadd_(name, list, lenlis )
char *name;
u_short *list;
int *lenlis; {
    return( CKADD( name, list, *lenlis )); }
int CKADD(name, list, lenlis)
char *name;
u_short *list;
int lenlis;
{
    message.command = (u_short)CC_CMD_ADDKLIST;
    message.klname = name;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    vec[1].iov_base = (caddr_t)list;
    vec[1].iov_len = lenlis * sizeof(u_short);
    errno = 0;
    writev(cc_path, vec, 2);
    return errno;
}

/* CKEXEC */
int ckexec_(name, buf, lenbuf, iiosb )
char *name;
u_short *buf;
int *lenbuf;
struct cc_iosb *iiosb; {
    return( CKEXEC( name, buf, *lenbuf, iiosb ) ); }
int CKEXEC(name, buf, lenbuf, iiosb)
char *name;
u_short *buf;
int lenbuf;
struct cc_iosb *iiosb;
{
    message.command = (u_short)CC_CMD_EXEKLIST;
    message.ptr_iosb = iiosb;
    message.klname = name;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    vec[1].iov_base = (caddr_t)buf;
    vec[1].iov_len = lenbuf * sizeof(u_short);
    errno = 0;
    writev(cc_path, vec, 2);
    check_file_pointer(vec[0].iov_len + vec[1].iov_len);
    return errno;
  }

/* CKSAVE */
int cksave_(fname)
char *fname; {
    return( CKSAVE( fname ) ); }
int CKSAVE(fname)
char *fname;
{
    register int fd;

    message.command = (u_short)CC_CMD_SAVEKLIST;
    message.ptr_data = klist;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    errno = 0;
    if( writev(cc_path, vec, 1) < 0 )
        return errno;
    fd = creat(fname, 0666);
    if( write(fd, &klist[0], 2) < 0 )
        return errno;
    if( write(fd, &klist[1], klist[0] * 2) < 0 )
        return errno;
    close(fd);
    return errno;
}

/* CKLOAD */
int ckload_(fname)
char *fname; {
    return( CKLOAD( fname ) ); }
int CKLOAD(fname)
char *fname;
{
    register int fd;

    fd = open(fname, O_RDONLY);
    if( read(fd, &klist[0], 2) < 0 )
        return errno;
    if( read(fd, &klist[1], klist[0] * 2) < 0 )
        return errno;
    close(fd);
    message.command = (u_short)CC_CMD_LOADKLIST;
    message.ptr_data = klist;
    vec[0].iov_base = (caddr_t)&message;
    vec[0].iov_len = sizeof(struct message);
    errno = 0;
    writev(cc_path, vec, 1);
    return errno;
  }

int check_file_pointer(bytecount) int bytecount; {

    file_pointer += bytecount;
    if( file_pointer > 0x70000000 ) {
        lseek(cc_path, SEEK_SET, 0);
        file_pointer = 0;
    }
}




