/* File name     : vmelib.c
 * Creation date : May 1993
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Modified      : August 1994, Y.Yasu
 *               : October 1994, Y.Yasu
 *               :   fixed a bug
 */

#include	<stdio.h>
#include	<fcntl.h>
#include	<sys/types.h>
#include	<sys/file.h>
#include	<sys/mman.h>
#include        <sys/ioctl.h>
#include        <sys/errno.h>
#include        "vmelib.h"
#define         SLOT 6
/* SLOT = 0 for VME_A16D16,
   SLOT = 1 for VME_A16D32,
   SLOT = 2 for VME_A24D16,
   SLOT = 3 for VME_A24D32,
   SLOT = 4 for VME_A32D16,
   SLOT = 5 for VME_A32D32,
*/
extern	int	errno;

/* For VME devices */
static int vme_fd[SLOT];
static int vme_offset[SLOT];
static int vme_size[SLOT];

/* For Interrupt handler */
int     vme_path;
static struct vmecmd vmecmd;

volatile caddr_t vme_mapopen(mode, vmeaddr, size)
int mode;
volatile caddr_t vmeaddr;
size_t size;
{
    volatile caddr_t vme_vaddr;
    int slot;
    char vme_dev[15];
    int vme_mapsize;
    int vme_pagesize;

    errno = 0;
    vme_pagesize = getpagesize();
    switch (mode) {
    case VME_A16D16MODE :
#ifdef DEBUG
        printf("VME_A16D16MODE\n");
#endif
	slot = 0;
        if( size > VME_A16D16SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A16D16BASE) || 
	    (vmeaddr > (volatile caddr_t)(VME_A16D16BASE+VME_A16D16SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr&0xFFFF - VME_A16D16BASE;
        strcpy(vme_dev,"/dev/vmea0");
	break;
    case VME_A16D32MODE :
#ifdef DEBUG
        printf("VME_A16D32MODE\n");
#endif
	slot = 1;
        if( size > VME_A16D32SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A16D32BASE) || 
	    (vmeaddr > (volatile caddr_t)(VME_A16D32BASE+VME_A16D32SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr&0xFFFF - VME_A16D32BASE;
        strcpy(vme_dev,"/dev/vmeb0");
	break;
    case VME_A24D16MODE :
#ifdef DEBUG
        printf("VME_A24D16MODE\n");
#endif
	slot = 2;
        if( size > VME_A24D16SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A24D16BASE) || 
	    (vmeaddr > (volatile caddr_t)(VME_A24D16BASE+VME_A24D16SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr&0xFFFFFF - VME_A24D16BASE;
        strcpy(vme_dev,"/dev/vmec0");
	break;
    case VME_A24D32MODE :
#ifdef DEBUG
        printf("VME_A24D32MODE\n");
#endif
	slot = 3;
        if( size > VME_A24D32SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A24D32BASE) || 
	    (vmeaddr > (volatile caddr_t)(VME_A24D32BASE+VME_A24D32SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr&0xFFFFFF - VME_A24D32BASE;
        strcpy(vme_dev,"/dev/vmed0");
	break;
    case VME_A32D16MODE :
#ifdef DEBUG
        printf("VME_A32D16MODE\n");
#endif
        slot = 4;
        if( size > VME_A32D16SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A32D16BASE) || 
	    (vmeaddr > (volatile caddr_t)(VME_A32D16BASE+VME_A32D16SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr - VME_A32D16BASE;
        strcpy(vme_dev,"/dev/vmee0");
	break;
    case VME_A32D32MODE :
#ifdef DEBUG
        printf("VME_A32D32MODE\n");
#endif
        slot = 5;
        if( size > VME_A32D32SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A32D32BASE) || 
	    (vmeaddr > (volatile caddr_t)(VME_A32D32BASE+VME_A32D32SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr - VME_A32D32BASE;
        strcpy(vme_dev,"/dev/vmef0");
	break;
    };
    vme_mapsize = size + vme_offset[slot]%vme_pagesize;
    vme_size[slot] = vme_mapsize;
    if ((vme_fd[slot] = open(vme_dev, O_RDWR)) == -1)
        return (volatile caddr_t)-1;
    /* if mmap fails, return value becomes -1 */
    vme_vaddr = (caddr_t)mmap(0, vme_mapsize, PROT_READ | PROT_WRITE,
			      MAP_SHARED, vme_fd[slot],
			      vme_offset[slot]/vme_pagesize*vme_pagesize );
#ifdef DEBUG
    printf("vme_vaddr = %x\n", vme_vaddr);
    printf("vme_offset = %x\n", vme_offset[slot]);
    printf("vme_pagesize = %x\n", vme_pagesize);
#endif
    if (vme_vaddr == (volatile caddr_t) -1) {
#ifdef DEBUG
        printf("errno = %d\n", errno);
#endif
        return (volatile caddr_t)-1;
    } else
        return (vme_vaddr + vme_offset[slot]%vme_pagesize);
}

caddr_t vme_mapclose(addr, mode)
caddr_t addr;
int mode; {
    int status, vme_vaddr, slot;

    errno = 0;
    switch (mode) {
    case VME_A16D16MODE :
        slot = 0;
        break;
    case VME_A16D32MODE :
        slot = 1;
        break;
      case VME_A24D16MODE :
        slot = 2;
        break;
      case VME_A24D32MODE :
        slot = 3;
        break;
      case VME_A32D16MODE :
        slot = 4;
        break;
      case VME_A32D32MODE :
        slot = 5;
        break;
      }
    vme_vaddr = (int)addr - vme_offset[slot]%getpagesize();
    status = munmap(vme_vaddr, vme_size[slot]);
    if( status == -1 ) {
      return (caddr_t)-1;
    }
    close(vme_fd[slot]);
    return 0;
}

int vme_mapprobe( mm, size, mode )
volatile char *mm;
int size, mode; {
    int i, temp, *ptrl;
    short *ptrw;
    char *ptrc;

    switch ( mode )  {
    case VME_A16D32MODE :
    case VME_A32D32MODE :
    case VME_A24D32MODE :
        ptrl = ( long *)mm;
	for( i = 0; i < size/sizeof(long); i++ )
	     temp = *ptrl++;
    case VME_A16D16MODE :
    case VME_A24D16MODE :
    case VME_A32D16MODE :
        ptrw = ( short *)mm;
	for( i = 0; i < size/sizeof(short); i++ )
	     temp = *ptrw++;
        ptrc = ( char *)mm;
	for( i = 0; i < size/sizeof(long); i++ )
	    temp = *ptrc++;
	break;
    }
}

int vme_intopen( vector, intrno )
int vector;
int *intrno; {

    errno = 0;
    if ((vme_path = open("/dev/vmea0", O_RDWR)) == -1)
        return errno;
    vmecmd.intrno = vector;
#ifdef DEBUG
    printf("vme_intopen: open ok\n");
    printf("vme_intopen: vector num = %x\n", vector);
#endif
    if( (ioctl(vme_path, VMEIOC_OPEN, &vmecmd )) == -1 )
        return errno;
    *intrno = vmecmd.data;
    return 0;
}

int vme_intclose( intrno )
int intrno; {

    errno = 0;
    vmecmd.intrno = intrno;
    if( ( ioctl( vme_path, VMEIOC_CLOSE, &intrno ) ) == -1 )
        return errno;
    return 0;
}


int vme_intenable( intrno )
int intrno; {
    int intrno1 = intrno;

    errno = 0;
    if( ( ioctl( vme_path, VMEIOC_ENABLE, &intrno1 ) ) == -1 )
        return errno;
    return 0;
}

int vme_intdisable( intrno )
int intrno; {
    errno = 0;
    if( ( ioctl( vme_path, VMEIOC_DISABLE, &intrno ) ) == -1 )
        return errno;
    return 0;
}

int vme_intwait( intrno, interrupt_count, timeout )
int intrno, *interrupt_count, timeout; {

    errno = 0;
    vmecmd.intrno = intrno;
    vmecmd.data = timeout;
    ioctl( vme_path, VMEIOC_WAIT, &vmecmd );
#ifdef DEBUG
    printf("vmecmd.reply1 = %d\n", vmecmd.reply1);
    printf("vmecmd.reply2 = %d\n", vmecmd.reply2);
#endif
    *interrupt_count = vmecmd.reply1;
    if( errno == EINTR && vmecmd.reply2 == -1 )
        return ETIMEDOUT;
    if( errno == EINTR && vmecmd.reply2 == -2 )
        return errno;
    if( errno == EINTR && vmecmd.reply2 == 0 )
        return 0;
    return errno;
}

int vme_intnowait( intrno, interrupt_count )
int intrno, *interrupt_count; {

    errno = 0;
    vmecmd.intrno = intrno;
    ioctl( vme_path, VMEIOC_NOWAIT, &vmecmd );
#ifdef DEBUG
    printf("vmecmd.reply1 = %d\n", vmecmd.reply1);
    printf("vmecmd.reply2 = %d\n", vmecmd.reply2);
#endif
    *interrupt_count = vmecmd.reply1;
    if( errno == EINTR && vmecmd.reply2 == 0 )
        return 0;
    return errno;
}

int vme_intclear( intrno )
int intrno; {
   if( ( ioctl( vme_path, VMEIOC_CLEAR, &intrno ) ) == -1 )
        return errno;
}

int vme_intcheck( intrno, interrupt_count )
int intrno, *interrupt_count; {

    errno = 0;
    vmecmd.intrno = intrno;
    ioctl(vme_path, VMEIOC_CHECK, &vmecmd );
#ifdef DEBUG
    printf("vmecmd.reply1 = %d\n", vmecmd.reply1);
    printf("vmecmd.reply2 = %d\n", vmecmd.reply2);
#endif
    *interrupt_count = vmecmd.reply1;
    if( errno == EINTR && vmecmd.reply2 == 0 )
        return 0;
    return errno;
}

