/* File name     : ccreg.h
 * Creation date : August 1991
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Modified date : August 1992, Y.Y
 */

struct CAMAC {
    volatile u_short CSER,D1;   /* Channel Status/Error Register */
    volatile u_short DOCR;      /* Device/Operation Control Register */
    volatile u_short SCCR,D2;   /* Sequence/Channel Control Register */
    volatile u_short MTCR;      /* Memory Transfer Count Register */
    volatile u_short MACHI;     /* Memory Address Counter High */
    volatile u_short MACLO;     /* Memory Address Counter Low */
    u_short dummy1[0x30/sizeof(u_short)];
    volatile u_short LAMICR;    /* LAM Interrupt Control Register */
    volatile u_short DONICR;    /* Done Interrupt Control Register */
    volatile u_short EMPICR;    /* DMA Buffer Empty Interrupt Control Reg. */
    volatile u_short ABOICR;    /* List Abort Interrupt Control Register */
    volatile u_short LAMIVR;    /* LAM Interrupt Vector Register */
    volatile u_short DONIVR;    /* Done Interrupt Vector Register */
    volatile u_short EMPIVR;    /* DMA Buffer Empty Interrupt Vector Reg. */
    volatile u_short ABOIVR;    /* List Abort Interrupt Vector  Register */
    u_short dummy2[0x10/sizeof(u_short)];
    volatile u_short AMR;       /* Address Modifier Register */
    volatile u_short CMR;       /* Command Memory Register */
    volatile u_short CMAR;      /* Command Memory Address Register */
    volatile u_short CWCR;      /* Command Word Counter Register */
    volatile u_short SRR;	/* Service Request Register */
    volatile u_short DLR;	/* Data Low Register */
    volatile u_short DHR;	/* Data High Register */
    volatile u_short CSR;  	/* Control/Status Register */
};

/* CSER field */
#define CC_CSERM_COC    0X8000  /* Channel Operation Complete */
#define CC_CSERM_NDT    0x2000  /* Normal Device Terminate */
#define CC_CSERM_ERR    0X1000  /* Error */
#define CC_CSERM_ACT    0x800   /* Channel Active */
#define CC_CSERK_SUCCES 0       /* Error Code : success */
#define CC_CSERK_BUSERR 0x9     /* Error Code : BUS error */
#define CC_CSERK_ABORT  0x11    /* Error Code : Software Abort */

/* DOCR field */
#define CC_DOCRK_INIT   0X8010  /* Cycle Steal, CAMAC Write, short word size */
#define CC_DOCRM_DIR    0X80    /* Direction : */
#define CC_DOCRK_READ   0X80    /* VME to CAMAC  = 0, CAMAC to VME = 1 */

/* SCCR field */
#define CC_SCCRM_START 0X80     /* Start Operation */
#define CC_SCCRM_ABORT 0X10     /* Software Abort */

/* LAMICR, DONICR, EMPICR, ABOICR field */
#define CC_INTRM_FLAG  0X80     /* Flag bit */
#define CC_INTRM_FAC   0X20     /* Flag auto clear */
#define CC_INTRM_EN    0X10     /* Interrupt Enable */
#define CC_INTRM_IAC   8        /* Interrupt auto clear bit */
#define CC_INTRK_IRQ4  4        /* Interrupt Request Level */ 
#define CC_INTK_TIMOUT 1
#define CC_INTK_LAM    2
#define CC_INTK_DONE   4
#define CC_INTK_EMPTY  8
#define CC_INTK_ABORT  0X10
#define CC_INTK_NOINT  0x20
#define CC_INTK_EN_IAC_FLG (CC_INTRM_EN | CC_INTRM_IAC | CC_INTRM_FLAG)

/* LAMIVR, DONIVR, EMPIVR, ABOIVR field */
#define CC_VECTRK_LAM   0XFF    /* LAM interrupt vector nummber */
#define CC_VECTRK_DONE  0XFF    /* DONE interrupt vector nummber */
#define CC_VECTRK_EMPTY 0XFF    /* BUFFER EMPTY interrupt vector nummber */
#define CC_VECTRK_ABORT 0XFF    /* LIST ABORT interrupt vector nummber */

/* AMR field */
/*#define CC_AMRK_INIT     0X79 */   /* short word, A24 */
                                /* Standard Non Priv. Data Access */
/*#define CC_AMRK_INIT     0X7D*/   /* short word, A24 */
                                /* Standard Super. Data Access */
#define CC_AMRK_INIT     0X49   /* short word, A32 */
#define CC_VME_SPACE  (A16+DATA_ACCESS)
#define CC_CAMAC_ADDR 0xFF00

/* Command Memory Address */
#define CC_CMAK_SINGLE  8189    /* CMA for single action */
#define CC_CMAK_SINGLEND 8191    /* CMA for single/block action */
#define CC_CMAK_BLOCK  8187    /* CMA for block action */
#define CC_CMAK_KLIST 0         /* CMA for Klist */
#define CC_CMAK_KLISTEND 8186   /* CMA for end of Klist */

/* Command Format */
#define CC_CMDM_AD      1       /* Abort Disable Bit */
#define CC_CMDM_WS      2       /* Word Size */
#define CC_CMDK_WS16    2       /* 16 bit */
#define CC_CMDK_WS24    0       /* 24 bit */
#define CC_CMDM_QM      8       /* Q mode */
#define CC_CMDK_QSTP    0       /* Q Stop */
#define CC_CMDK_IGNQ    8       /* Q Ignore */
#define CC_CMDK_RPRT    0X10    /* Q Repeat */
#define CC_CMDK_ASCN    0X18    /* Q Scan */
#define CC_CMDM_TM      0X20    /* Transfer mode */
#define CC_CMDM_CM      0X80    /* Command mode */
#define CC_CMDK_SINGLE  0X00    /* Singel CAMAC Action */
#define CC_CMDK_BLOCK   0X20    /* Block CAMAC Action */
#define CC_CMDK_INLWRT  0X60    /* Single Inline Write Action */
#define CC_CMDK_HALT    0X80    /* HALT instruction */
#define CC_CMDM_CRT     0X100   /* Crate number */
#define CC_CMDM_F       0       /* Functioin */
#define CC_CMDK_F8      8       /* F = 8 */
#define CC_CMDK_F16     0x10    /* F = 16 */
#define CC_CMDK_F24     0x18    /* F = 24 */
#define CC_CMDM_A       0X20    /* Subaddress */
#define CC_CMDM_N       0X200   /* Station number */
#define CC_CMDK_BLK_QS_16 (CC_CMDK_BLOCK | CC_CMDK_QSTP | CC_CMDK_WS16)
#define CC_CMDK_BLK_QS_24 (CC_CMDK_BLOCK | CC_CMDK_QSTP | CC_CMDK_WS24)
#define CC_CMDK_BLK_AS_16 (CC_CMDK_BLOCK | CC_CMDK_ASCN | CC_CMDK_WS16)
#define CC_CMDK_BLK_AS_24 (CC_CMDK_BLOCK | CC_CMDK_ASCN | CC_CMDK_WS24)
#define CC_CMDK_BLK_IG_16 (CC_CMDK_BLOCK | CC_CMDK_IGNQ | CC_CMDK_WS16)
#define CC_CMDK_BLK_IG_24 (CC_CMDK_BLOCK | CC_CMDK_IGNQ | CC_CMDK_WS24)

/* CSR field */
#define CC_CSRM_GO	1	/* Go DMA */
#define CC_CSRM_NOQ     2       /* NO Q */
#define CC_CSRM_NOX     4       /* NO X */
#define CC_CSRM_NOQX    0XFFF9  /* NO Q and X */
#define CC_CSRM_DIR     0X20    /* Direction : CAMAC->VME = 0 */
                                /*             VME->CAMAC = 1 */
#define CC_CSRM_WRITE   0x20    /* VME->CAMAC */
#define CC_CSRM_DMA     0X40    /* Direct Memory Access for Data Transfer */
                                /* VME Master = 1, VME Slave = 0 */
#define CC_CSRM_DONE	0X80	/* Completion Status */
#define CC_CSRM_RDY	0X100	/* Data Ready */
#define CC_CSRM_LAM	0X200	/* LAM is present. */
#define CC_CSRM_TM      0X400   /* Transfer Mode */
#define CC_CSRM_TM1     0X400   /* Transfer Mode bit 0 */
#define CC_CSRM_TM2     0X800   /* Transfer Mode bit 1 */
#define CC_CSRM_RST	0X1000	/* Reset the controller */
#define CC_CSRM_TMO     0X2000  /* Time Out */
#define CC_CSRM_ABT     0X4000  /* Abort : ERROR & Abort Disable bit = 0 */
#define CC_CSRM_ERR     0X8000  /* error */
#define CC_CSRK_EMASK   0XF006  /* Bit15:12 & Bit2:1 */
#define CC_CSRK_RD_ER   (CC_CSRM_RDY|CC_CSRM_ERR)
#define CC_CSRK_DN_ER   ( CC_CSRM_DONE | CC_CSRM_ERR )
#define CC_CSRK_WR_GO   (CC_CSRM_GO | CC_CSRM_WRITE)
