/*
 * bufmgr.c
 *
 * Nova buffer manager  program
 *              version 0.33  28-AUG-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.01  25-SEP-1992 Masaharu NOMACHI (KEK)
 *		version 1.07	20-NOV-1993	Yasuo TAKEUCHI	(T.I.T.)
 */

#include <errno.h>
#include "novad.h"
#include "bm.h"

#define MS_install  101
#define MS_remove   102

#define MS_done     111

#define MS_pause    121
#define MS_resume   122
#define MS_request  123

#define MS_dump     140
#define MS_debug    141

#define MS_exit_bm  999

/***********************
 * buffer manager main *
 ***********************/

buffer_manager(p) struct nova_def *p; {
  struct msgbuf *msg_buffer;
  int rval, i,j,k;

  msg_buffer = (struct msgbuf *)malloc(sizeof(long) + NV_MSGLEN);
  if ((int)msg_buffer == -1) {
    perror("buffer manager malloc");
    return -1;
  }

  if (com_init() < 0) return -1;
  if (exec_init(NV_THRESHOLD) < 0) return -1;
  if (init_data_base(p) < 0) return -1;

  /*
   * Wait message
   */
  while(1) {
    errno = 0;
    rval = receive_bufmgr(p,msg_buffer);
    if (rval<0) continue;

    rval = exam_message(msg_buffer);
    switch(rval) {
    case MS_request:
      rval = nova_request(p,msg_buffer);
      break;
    case MS_done:
      rval = nova_done(p,msg_buffer);
      break;
    case MS_resume:
      rval = nova_resume1(p,msg_buffer);
      break;
    case MS_pause:
      rval = nova_pause1(p,msg_buffer);
      break;
    case MS_install:
      rval = nova_install(p,msg_buffer);
      break;
    case MS_remove:
      rval = nova_remove(p,msg_buffer);
      break;
    case MS_dump:
      rval = nova_dump(p);
      break;
    case MS_debug:
      rval = nova_dump(0);
      break;
    case MS_exit_bm:
      printf("Exit NOVA\n");
      exit(0);
      break;
    default:
      printf("bufmgr:Invalid message %s\n",msg_buffer->mtext);
      break;
    }
    if (rval) nova_error(p,msg_buffer);
  }
}
/****************************
 * Initialize the data base *
 ****************************/

init_data_base(p) struct nova_def *p; {
  
  /*  
   * printf(" I am a buffer manager. %d %d %d\n",
   *     p->total_size, p->n_buffer, p->buff_size);
   */
  return init_table(p->n_process,p->n_buffer,p->buff_size);
}


/************************************* 
 * nova_install()                    *
 *   add process queue data base     *
 *************************************/
int nova_install(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer;
{
  int pid, pri, msg, mtype, rval, request;
  struct BMT_process *ptable, *assign_ptable();

  rval = sscanf(msg_buffer->mtext,"install:%d:%d:%d:%d",
		&pid,&pri,&request,&msg);
  if (rval != 4) return -1;
  ptable = assign_ptable(p,pid);
  if (ptable==NULL_ptable) {
    sprintf(msg_buffer->mtext,"error:%d:0:too many processes",pid);
  }
  else {
    rval = set_client_msg(p,ptable->id,&msg,&mtype);
    rval = com_install(ptable,pid,msg,mtype);
    rval = exec_process_install(ptable,0, pri,request);
    sprintf(msg_buffer->mtext,"ack:%d:%d:(%d,%d):",
	    pid,ptable->id,msg,mtype);
  }
  send_daemon(p,msg_buffer);
  return 0;
}


/************************************ 
 * nova_remove()                    *
 *     close conection              *
 ************************************/

int nova_remove(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer;
{
  int rval, pid, novaid;
  struct BMT_process *ptable, *get_ptable();

  rval = sscanf(msg_buffer->mtext,"remove:%d:%d",&pid,&novaid);
  if (rval != 2) return -1;
  ptable = get_ptable(p,pid,novaid);
  if (ptable==NULL_ptable) return -1;
  exec_remove(ptable);
  com_remove(ptable);
  return 0;
}

/************************************ 
 * nova_pause1()                    *
 *     pause conection              *
 ************************************/

int nova_pause1(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer;
{
  int rval, pid, novaid;
  struct BMT_process *ptable, *get_ptable();

  rval = sscanf(msg_buffer->mtext,"pause+hold:%d:%d",&pid,&novaid);
  if (rval == 2) {
    ptable = get_ptable(p,pid,novaid);
    if (ptable==NULL_ptable) return -1;
    return bm_pause(ptable,"hold");
  }
  rval = sscanf(msg_buffer->mtext,"pause+done:%d:%d",&pid,&novaid);
  if (rval == 2) {
    ptable = get_ptable(p,pid,novaid);
    if (ptable==NULL_ptable) return -1;
    return bm_pause(ptable,"done");
  }
  return -1;
}

/************************************ 
 * nova_resume1()                   *
 *     resume conection             *
 ************************************/

int nova_resume1(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer;
{
  int rval, pid, novaid, request;
  struct BMT_process *ptable, *get_ptable();

  rval = sscanf(msg_buffer->mtext,"resume:%d:%d:%d",&pid,&novaid,&request);
  if (rval != 3) return -1;
  ptable = get_ptable(p,pid,novaid);
  if (ptable==NULL_ptable) return -1;
  return bm_resume(ptable,request);
}


/************************************ 
 * nova_request()                   *
 *     request one buffer           *
 ************************************/

int nova_request(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer;
{
  int rval, pid, novaid;
  struct BMT_process *ptable, *get_ptable();

  rval = sscanf(msg_buffer->mtext,"request:%d:%d:%d",&pid,&novaid);
  if (rval != 2) return -1;
  ptable = get_ptable(p,pid,novaid);
  if (ptable==NULL_ptable) return -1;
  return bm_request(ptable);
}

/*********************************** 
 * nova_done()                     *
 * release an active data buffer   *
 ***********************************/
int nova_done(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer;
{
  int rval;
  int pid,novaid,offset,size,sig;
  struct BMT_process *ptable, *get_ptable();
  struct BMT_buffer  *btable, *get_btable();

  rval = sscanf(msg_buffer->mtext,"done:%d:%d:%d:%d:%d",
		&pid,&novaid,&offset,&size,&sig);
  if (rval != 5) return -1;
  ptable = get_ptable(p,pid,novaid);
  btable = get_btable(p,offset);
  return bm_done(ptable,btable,0,size,sig);
}


/************************************ 
 * nova_error()                     *
 ************************************/

int nova_error(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer;
{
  int rval, pid, novaid;
  struct BMT_process *ptable, *get_ptable();

  printf("Error!! %s\n",msg_buffer->mtext);

  return 0;
}


/*
 ******************************************************************
 */

trg_buff_install() {
  return;
}

trg_install() {
  return;
}

trg_remove() {
  return;
}

int bm_req_test() {
  return 1;
}

int exam_message(msg_buffer) struct msgbuf *msg_buffer; {
  if      (strncmp("done",   msg_buffer->mtext,4) == 0) return MS_done;
  else if (strncmp("request",msg_buffer->mtext,7) == 0) return MS_request;
  else if (strncmp("resume", msg_buffer->mtext,6) == 0) return MS_resume;
  else if (strncmp("install",msg_buffer->mtext,7) == 0) return MS_install;
  else if (strncmp("remove", msg_buffer->mtext,6) == 0) return MS_remove;
  else if (strncmp("pause",  msg_buffer->mtext,5) == 0) return MS_pause;
  else if (strncmp("dump",   msg_buffer->mtext,4) == 0) return MS_dump;
  else if (strncmp("debug",  msg_buffer->mtext,5) == 0) return MS_debug;
  else if (strncmp("exit_bm",msg_buffer->mtext,7) == 0) return MS_exit_bm;
  else                                                  return -1;
}
