/*
 * novad_exam_arg.c
 *
 *    Examine the arguments for novad
 *                                    KEK online group
 *                                    Masaharu NOMACHI
 *                                    Tokyo Institute of Technology
 *                                    Yasuo TAKEUCHI
 *
 *              version 2.00  21-feb-1994 Masa NOMACHI
 *              version 1.07  27-NOV-1993 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.06  12-APR-1993 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.05  24-NOV-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.01  26-SEP-1992 Masaharu NOMACHI
 *              version 0.33  28-AUG-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 0.2   08-JUN-1992 Masaharu NOMACHI
 *
 *  novad [options] name
 *       -b #     buffer size in K-byte
 *       -n #     number of buffers
 *       -p #     max process number
 */

#include "novad.h"

show_usage()
{
    printf("\n");
    printf("  NOVA Buffer Manager version 2.00 21-FEB-1994 ");
    printf("written by M.Nomachi, Y.Takeuchi\n");
    printf("  usage: novad [options] name [options] \n");
    printf("\n");
    printf("           -b #    buffer size in K-byte\n");
    printf("           -n #    number of buffers\n");
    printf("           -p #    max number of client processes\n");
    printf("           -h #    show usage\n");
    printf("\n");

    return 0;
}

int exam_arg(argc,argv,p) int argc; char *argv[]; struct nova_def *p; {
  int i;
  char *nova_name, *getenv();
  
  p->buff_size = (16<<10); /* buffer size = 16K byte          */
  p->n_buffer  = 8;        /* default 8 buffer.               */
  p->n_process = 20;       /* default max 20 process          */
  p->name[0] = '\0';
  
  for (i = 1; i < argc; i++) {
    if (argv[i][0] == '-') {
      if (argv[i][1] == 'b') {
	i++; p->buff_size = atoi(argv[i]); p->buff_size <<= 10;
      }
      else if (argv[i][1] == 'n') {
	i++; p->n_buffer = atoi(argv[i]);
      }
      else if (argv[i][1] == 'p') {
	i++; p->n_process = atoi(argv[i]);
      }
      else if (argv[i][1] == 'h') {
	show_usage();
	exit(1);
      }
    }
    else if (strlen(argv[i]) <= NV_MAXNAME)
      strcpy(p->name, argv[i]);
  }
  
  if (p->name[0] == '\0') {
    nova_name = getenv("NOVA");
    if ((int)nova_name == 0) {
      show_usage();
      exit(1);
    }
    strcpy(p->name, nova_name);
  }

#ifdef VXWORKS
  if (p->name[0] == '\0')  {
    show_usage();
    exit(1);
  }
#endif
  p->total_size = p->buff_size * p->n_buffer;
  
  if ((p->total_size > NV_MAXTOTAL) || (p->total_size < 1)) {
    printf("Invalid shared memory size %d\n", p->total_size);
    return -1;
  }
  if (p->buff_size < 1) {
    printf("Invalid size of buffer %d\n", p->buff_size);
    return -1;
  }
  if (p->n_buffer < 1) {
    printf("Invalid number of buffer %d\n", p->n_buffer);
    return -1;
  }
  if (p->n_process < 1) {
    printf("Invalid number of max process %d\n", p->n_process);
    return -1;
  }
  
  return 0;
}








