/*
 *  This is file UNI_nova.c
 *  R. Ball, March, 1993
 *  File purpose is to isolate calls to NOVA from the user, such that
 *  machine-independence of "analyzer" programs can be maintained.  This
 *  means that, if the NOVA event source is on a remote machine, the net-nova
 *  routines will be called, and if the source is local, the local-nova
 *  routines will be called, all transparently to the user.
 *  DEBUG switch may be turned on during compilation if needed.
 */

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef VXWORKS
#include <vxWorks.h>
#include <msgQLib.h>
#include "vxenvvar.h"
char *getenv();
#else
#include <stdio.h>
#include <stdlib.h>
#endif
#include "nova.h"

struct NOVAD *net_nova_open(  );
extern char Nova_Host[];
extern int nova_prio;

/**********************************************************************/
/*   Routine UNI_nova_parms ( d_name, d_prio, p_name, s_name)         */
/*    This routine takes a passed default NOVA socket name and        */
/*    priority, returns the primary and secondary socket names, and   */
/*    fills the nova_prio global variable with the local priority,    */
/*    either the default or that passed on the command line via the   */
/*    -p option.                                                      */
/*    If Nova_Host is NULL, p_name is the same as s_name.             */
/*    If Nova_Host is not NULL, p_name is the remote socket.          */
/**********************************************************************/
UNI_nova_parms(d_name, d_prio, p_name, s_name)
char *d_name, *p_name, *s_name;
int d_prio;
{
  char *env_val;
  if((env_val = getenv("LOCAL_NOVA")) == NULL)  strcpy(s_name, d_name);
  else                                          strcpy(s_name, env_val);

  if((env_val = getenv("REMOTE_NOVA")) == NULL) strcpy(p_name, d_name);
  else                                          strcpy(p_name, env_val);

  if (Nova_Host[0] == '\0') {
    strcpy(p_name, s_name);
  }
  else   {
    strcat (p_name, "@");
    strcat (p_name, Nova_Host);
  }

  if (nova_prio == -99)  nova_prio = d_prio;
}

/**********************************************************************/
/*   Routine UNI_nova_open ( name, prio, Nreq )                       */
/*   This routine interfaces the user code to either nova_open        */
/*     or net_nova_open, depending on the state of name.              */
/*       name=socket is local NOVA.                                   */
/*       name=socket@machine or name=socket@machine:port is remote.   */
/**********************************************************************/
struct NOVAD *UNI_nova_open ( name, prio, Nreq)
char *name;
int  prio, Nreq;
{
  char *host_name, *port_number, *index();
  int port;
  struct NOVAD *nd;

  host_name = index(name,'@');
  if ((int)host_name==0) return nova_open(name,prio,Nreq);
  else {
    *host_name++ = '\0';
    port_number = index(host_name,':');
    if ((int)port_number==0) port = 0;
    else {
      *port_number++ = '\0';
      port = atoi(port_number);
    }
    nd = net_nova_open(host_name, port, name, prio, Nreq);
    if ((int)nd==-1) return nd;
    nd->base = (char *) malloc(nd->max_length);
    return nd;
  }
}

/**********************************************************************/
/*   Routine UNI_nova_get ( nd, data, len )                           */
/*   Pick up a nova event buffer, either via pointer locally or       */
/*     from a machine remotely via IP.                                */
/**********************************************************************/
int UNI_nova_get ( nd, data, len )
struct NOVAD *nd;
char   **data;
int    *len;
{
  int status;
  int r_value;
  char **PPTb;

  if ( (int)nd == -1 )  return(-1);
  if (nova_is_local(nd)) return (nova_get(nd, data, len));
  else {
    status = net_nova_read ( nd, nd->base, nd->max_length);
    if (status == 0)  {
      *len = 0;
      return 1;
    }
    else if (status < 0)  {
      *len = 0;
      return 2;
    }
    else {
      *len = status;
      *data = nd->base;
/*
 *  For buffers received over the net, ensure the Endian-ism is the same.
 *  If it is not, invert the words of the buffer.
 */
      PPTb=data;      
      check_buffer_hibytes(*PPTb);
      return 0;
    }
  }
}

/************************************************************************/
/*   Routine check_buffer_hibytes(char *);                              */
/*   If first integer of buffer (buffer length) is <0 or >48K then      */
/*      buffer is from a machine with inverted byte order, so invert    */
/*      all bytes in buffer.                                            */
/*      Maybe instead of 48K we should use nd->max_length...is that     */
/*        the max buffer length                                         */
/************************************************************************/
check_buffer_hibytes(buf)
char *buf;
{
	int *test_length, true_length, j;
	char *PTb;		/* buffer pointer */
	char *tbuf;             /* pointer to next event within buf */
	char temp[4];		/* mini buffer to invert bytes */
	char *PTs, *PTd;	/* pointer to source, destination */

	test_length = (int *) buf;
#ifdef DEBUG
	printf ("START: test_length address is %d\n",test_length);
#endif
	while (*test_length != 0) {
	  if (*test_length<0 || *test_length>0x30000)
	  {
		
	        tbuf = (char *) test_length;
	     /* copy first integer into temp */
		for (PTs=tbuf, PTd=temp; PTs<tbuf+4; PTs++, PTd++) 
		  *PTd = *PTs;
	     /* copy back to buffer, but invert bytes */
		for (PTs=temp, PTd=tbuf+3; PTs<temp+4; PTs++, PTd--) 
		  *PTd = *PTs;

		true_length = *test_length - 5;
#ifdef DEBUG
		printf ("       true_length is %d\n",true_length);
#endif
	     /* loop through entire buffer, but only if it is not BEGIN,
	                                           PAUSE, RESUME or END  */
		PTb=tbuf+4;
		for (j=5; j>0; j--)
		{
			for (PTs=PTb, PTd=temp; PTs<PTb+4; PTs++, PTd++) 
			  *PTd = *PTs;
			for (PTs=temp, PTd=PTb+3; PTs<temp+4; PTs++, PTd--) 
			  *PTd = *PTs;
			PTb+=4;
		}

		if ( *(test_length+1)<1 || *(test_length+1)>4 )
		{
#ifdef DEBUG
		   printf ("       Doing remainder of event...\n");
#endif
		   while (--true_length)
		   {
			for (PTs=PTb, PTd=temp; PTs<PTb+4; PTs++, PTd++) 
			  *PTd = *PTs;
			for (PTs=temp, PTd=PTb+3; PTs<temp+4; PTs++, PTd--) 
			  *PTd = *PTs;
			PTb+=4;
		   }
	        }
	  }
	  test_length += *test_length;
#ifdef DEBUG
	  printf ("       New test_length address and content are %d and %d\n",
		  test_length, *test_length);
#endif
	}
}

/**********************************************************************/
/*   Routine UNI_nova_put ( nd, address, size, sig )                  */
/*   Put a nova buffer back into the chain of those being passed.     */
/*     If local, use nova_put.  If remote, no action is required.     */
/**********************************************************************/
int UNI_nova_put ( nd, address, size, sig )
struct NOVAD *nd;
char   *address;
int    size, sig;
{
  if ( (int)nd == -1 )  return(-1);
  if (nova_is_local(nd)) return nova_put(nd,address,size,sig);
  else  return 0;
}

/**********************************************************************/
/*   Routine UNI_nova_close ( nd )                                    */
/*   Close nova.  For the remote case, free the malloc() data space.  */
/**********************************************************************/
int UNI_nova_close ( nd )
struct NOVAD *nd;
{
  if ( (int)nd == -1 )  return(-1);
  if (nova_is_local(nd)) return nova_close(nd);
  else  {
    free(nd->base);
    return (net_nova_close(nd));
  }
}






