/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      June 20, 1992                                            */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef VXWORKS
#include "vxshm.h"
#define key_t int
#define memcpy(s1,s2,i) bcopy(s2,s1,i)
#else
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdlib.h>
#include <errno.h>
#endif
#include "template.h"

static int  shmid_table, shmid_header;
static Table  *shmaddr_table;
static Header *shmaddr_header;

/*(a+out) ------------------------------------------------- */
int Create_Table()
/*(b+out) -------------------------------------------------
Procedure Name
Creat_Table - creates the Common Table.
Synopsis
Description
Creat_Table(process) creates the Common Table (CT). 
process: the process for which the CT to be initialized.
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   key_t key;
   int   size, shmflg;

   key = TABLE_KEY;
   size = TABLE_SIZE * sizeof(struct table);
   shmflg = TABLE_FLAG | IPC_CREAT;
   if ((shmid_table = shmget(key, size, shmflg)) == -1) return ERROR;

   key = HEADER_KEY; 
   size = sizeof(Header);
   shmflg = TABLE_FLAG | IPC_CREAT;
   if ((shmid_header = shmget(key, size, shmflg)) == -1) return ERROR;

   if ((shmaddr_table = (struct table *)shmat(shmid_table, 0, 0))
      == (struct table *)-1) return ERROR;
   if ((shmaddr_header = (Header *)shmat(shmid_header, 0, 0)) 
      == (Header *)-1) return ERROR;

   shmaddr_header->first = shmaddr_header->last = 0; 

   return SUCCESS;
}




/*(a+out) ------------------------------------------------- */
int Attach_Table()
/*(b+out) -------------------------------------------------
Procedure Name
Attach_Table - initializes the Common Table
Synopsis
Description
Attach_Table() initializes the Common Table (CT). 
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   key_t key;
   int   size, shmflg;

   key = TABLE_KEY;
   size = TABLE_SIZE * sizeof(struct table);
   shmflg = TABLE_FLAG;
   if ((shmid_table = shmget(key, size, shmflg)) == -1) {
      shmflg = TABLE_FLAG | IPC_CREAT;
      if ((shmid_table = shmget(key, size, shmflg)) == -1) return ERROR; 
   }

   key = HEADER_KEY; 
   size = sizeof(Header);
   if ((shmid_header = shmget(key, size, shmflg)) == -1) return ERROR;

   if ((shmaddr_table = (struct table *)shmat(shmid_table, 0, 0))
      == (struct table *)-1) return ERROR;
   if ((shmaddr_header = (Header *)shmat(shmid_header, 0, 0)) 
      == (Header *)-1) return ERROR;

   if (shmflg == (TABLE_FLAG | IPC_CREAT)) {
     shmaddr_header->first = shmaddr_header->last = 0;
   }
   return SUCCESS;
}





/*(a+out) ------------------------------------------------- */
int Detach_Table()
/*(b+out) -------------------------------------------------
Procedure Name
Detach_Table - detaches the Command Table from a process
Synopsis
Description
Detach_Table() detaches the Command Table from a process. 
        
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   if (shmdt(shmaddr_table) == -1) return ERROR;
   if (shmdt(shmaddr_header) == -1) return ERROR;
   shmaddr_header = NULL;
   shmaddr_table = NULL;

   return SUCCESS; 
}





/*(a+out) ------------------------------------------------- */
int Delete_Table()
/*(b+out) -------------------------------------------------
Procedure Name
Delete_Table - detaches the Command Table from a process
Synopsis
Description
Delete_Table() detaches the Command Table from a process.
       
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   struct shmid_ds buf;

   if (shmctl(shmid_table, IPC_RMID, &buf) == -1) return ERROR;
   if (shmctl(shmid_header, IPC_RMID, &buf) == -1) return ERROR;
   shmaddr_header = NULL;
   shmaddr_table = NULL;

   return SUCCESS; 
}





/*(a+out) ------------------------------------------------- */
int Add_to_Table(entry)
Table *entry;
/*(b+out) -------------------------------------------------
Procedure Name
Add_to_Table - adds an entry to the Common Table 
Synopsis
Description
Add_to_Table(entry) adds an entry to the Common Table.
entry: the entry that will be added to the Common Table. 
       
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   char *table_ptr;
   Header hdr;
   int j;

   if(shmaddr_table == NULL) return ERROR;
   if (shmaddr_header->last >= TABLE_SIZE)  {
/*           Linear filling of entries was exceeded.  Find an entry marked
 *           dead, if there are any, and reuse it.  */
      if (( j=get_dead_CT_entry() ) == -1)  {
         return ERROR;
      }
      else  {
	 table_ptr = (char *)shmaddr_table + j*sizeof(struct table);
	 memcpy(table_ptr, (char *)entry, sizeof(struct table));
       }
   }
   else  {
      table_ptr = (char *)shmaddr_table +
                  (shmaddr_header->last)*sizeof(struct table);
      memcpy(table_ptr, (char *)entry, sizeof(struct table));

      (shmaddr_header->last)++;
   }
   return SUCCESS; 
}





/*(a+out) ------------------------------------------------- */
int Read_from_Table(entry, index)
Table *entry;
int index;
/*(b+out) -------------------------------------------------
Procedure Name
Read_from_Table - reads an entry from CT 
Synopsis
Description
Read_from_Table(entry, index) reads an entry from CT. 
entry: the structure to where the entry will be read, 
index: the index of the entry to be read. 
        
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   char *table_ptr;

   if(shmaddr_table == NULL) return ERROR;
   table_ptr = (char *)shmaddr_table + index*sizeof(struct table);
   memcpy((char *)entry, table_ptr, sizeof(struct table)); 

   return SUCCESS; 
}





/*(a+out) ------------------------------------------------- */
int Search_Table(process_name)
char *process_name;
/*(b+out) -------------------------------------------------
Procedure Name
Search_Table - searches CT for a process name 
Synopsis
Description
Search_Table(process_name) searches CT for a process name.
process_name: the name of the process that will be searched.
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   Table entry;
   char *table_ptr;
   int  flag, index;

   flag = 0; index = 0;
   if(shmaddr_table == NULL) return ERROR;
   while((flag == 0) & (index < shmaddr_header->last)) {
      table_ptr = (char *)shmaddr_table + index*sizeof(struct table);
      memcpy((char *)&entry, table_ptr, sizeof(struct table)); 
      if (strcmp(entry.process_name, process_name) == 0) flag = 1;
      else index++;
   }
   if (flag == 1) return index;
   else return -1;
}



/*(a+out) ------------------------------------------------- */
int get_dead_CT_entry()
/*(b+out) -------------------------------------------------
Procedure Name
get_dead_CT_entry - searches CT for a DEAD process
Synopsis
Description
get_dead_CT_entry() searches CT for a DEAD process.
It returns the index (beginning with zero) within the table of the entry.
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   Table entry;
   char *table_ptr;
   int  flag, index;

   flag = 0; index = 0;
   if(shmaddr_table == NULL) return ERROR;
   while((flag == 0) & (index < shmaddr_header->last)) {
      table_ptr = (char *)shmaddr_table + index*sizeof(struct table);
      memcpy((char *)&entry, table_ptr, sizeof(struct table)); 
      if (entry.status == DEAD) flag = 1;
      else index++;
   }
   if (flag == 1) return index;
   else return -1;
}



/*(a+out) ------------------------------------------------- */
int Write_to_Table(entry, index)
Table *entry;
int index;
/*(b+out) -------------------------------------------------
Procedure Name
Write_to_Table - writes an entry to CT
Synopsis
Description
Write_to_Table(entry, index) writes an entry to CT. 
entry: the structure that will be written to CT, 
index: the index of CT to where the entry will be written.
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   char *table_ptr;

   if(shmaddr_table == NULL) return ERROR;
   table_ptr = (char *)shmaddr_table + index*sizeof(struct table);
   memcpy(table_ptr, (char *)entry, sizeof(struct table));

   return SUCCESS; 
}




int Get_Table(new_entry)
Table new_entry[];
{
   Table entry,*t_entry;
   char *table_ptr, status[10];
   int  flag, index, counter = 0;

   flag = 0; index = 0;
   if(shmaddr_table == NULL) return ERROR;
   t_entry = new_entry;
   while((flag == 0) & (index < shmaddr_header->last)) {
      table_ptr = (char *)shmaddr_table + index*sizeof(struct table);
      memcpy((char *)t_entry, table_ptr, sizeof(struct table)); 
      t_entry++;
/*      memcpy((char *)&entry, table_ptr, sizeof(struct table)); 
      strcpy(new_entry[counter].process_name,  entry.process_name); 
      new_entry[counter].process_id = entry.process_id;
      new_entry[counter].msgq_id =  entry.msgq_id;
      new_entry[counter].status =  entry.status; */
      counter++;
      index++;
   }
   return(counter);
}

void change_id_number( new_pid, name)
int new_pid;
char *name;
{
   Table entry;
   char *table_ptr, status[10];
   int  flag, index, counter = 0;

   printf("HELLO\n");
   flag = 0; index = 0;
   if(shmaddr_table == NULL) return;
   while((flag == 0) & (index < shmaddr_header->last)) {
      table_ptr = (char *)shmaddr_table + index*sizeof(struct table);
      memcpy((char *)&entry, table_ptr, sizeof(struct table)); 
      if (entry.status == ALIVE) strcpy(status, "ALIVE");
      else if (entry.status == DEAD) strcpy(status, "DEAD");
      if (strcmp(name, entry.process_name) == 0) {
         entry.process_id = new_pid;
         return;
      } 
      counter++;
      index++;
   }
}
