/**********************************************************
 file: vxshm.c
 author: C. Timmermans
 purpose: simulate the UNIX shared memory calls in VxWorks
***********************************************************/
/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/
#ifdef VXWORKS
#include <memLib.h>
#include "vxipc.h"
#define key_t int

#define MAXSHM 50
char *vxshmad[MAXSHM] = {NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
			 NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
			 NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
			 NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
			 NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
int vxkey[MAXSHM];

int shmget (key,size,shmflg)
/********************************************
 compatible with UNIX shmget; however I did not 
 implement the different access permissions
 *********************************************/
int key,size,shmflg;
{
  int i;

  if(key != IPC_PRIVATE) {
    for(i=0;i<MAXSHM;i++) {
      if((vxkey[i] == key) && (vxshmad[i] != NULL)) {
	if((shmflg & IPC_EXCL) == 0) return(i);
	return(-1);
      }
    }
    if((shmflg & IPC_CREAT) == 0) return(-1);
  }
  for(i=0;i<MAXSHM;i++){
    if(vxshmad[i] == NULL) {
      vxshmad[i] = (char *)malloc(size);
      if(vxshmad[i] == NULL) return(-1);
      bzero(vxshmad[i],size);
      vxkey[i] = key;
      return(i);
    }
  }
}

int shmctl(shmid,cmd,buf)
/*********************************************
 compatible with UNIX shmctl. Struct shmid_ds is not 
 implemented correctly. Actually only IPC_RMID is 
 implemented as a command. No check is performed before 
 removing the memory !!
 *********************************************/
int shmid,cmd;
struct shmid_ds *buf;
{
  if((shmid < 0) || (shmid >= MAXSHM)) return(-1);
  switch (cmd) {
  case IPC_STAT:
    return(-1);
  case IPC_SET:
    return(-1);
  case IPC_RMID:
    if(vxshmad[shmid] != NULL) free(vxshmad[shmid]);
    vxkey[shmid] = 0;
    return(0);
  }
  return(-1);
}

char *shmat(shmid,shmaddr,shmflg)
/*************************************************
 compatible with UNIX shmat. the shmaddr and shmflg
 parameters are ignored.
 *************************************************/
int shmid;
char *shmaddr;
int shmflg;
{
  if((shmid < 0) || (shmid >= MAXSHM)) return((char *)-1);
  if(vxshmad[shmid] == NULL) return((char *)-1);
  return(vxshmad[shmid]);
}

int shmdt(shmaddr)
/******************************************************
 here fore compatibility; no action taken
 ******************************************************/
char *shmaddr;
{
  return(0);
}
#endif
