/*****************************************************
 * FILE: runpar.h
 * Author: C. Timmermans, University of Michigan
 *****************************************************/
#include <stdlib.h>
#include <stdio.h>

/*****************************************************
 * DATATIME determines how often the runpar data is 
 * updated (in seconds)
 *****************************************************/
#define DATATIME 3
/********************************************************
  Please enter conversion routines (integer->string) here.
  At this point they should be added as: 
             extern int routine();
  The coding for these routines should be done in file 
  runp_conv.c. This file contains the information on how 
  to write these routines, and some examples.
 ********************************************************/

extern int conv_time(); /* converts integer -> string */
extern int conv_stat(); /* converts status representation <--> string */


struct rp_def rp_but[] = {
/**********************************************************
  For each variable to be displayed the parameter display fields 
  must be defined below. 
  Each field has 6 entries, ordered as follows:
  char txt[20];         Comment to display in front of the value
  char variable[30];    Name of the variable to display
  char proc[30];        Process to get the variable from
  int type;             Type of variable. Possible types are:
                        INT_DEC            integer, shown in decimal
                        INT_HEX            integer, shown hexadecimal
                        INT_CONV           integer, converted to a string 
                                           using a special routine.
                        REAL               real values
                        STRING             character strings
  int (*conversion)();  Routine to convert variable. If there is a standard
                        way to convert the variable, this value should be 
                        (int (*)())NULL. Otherwise the conversion routine 
                        should be entered in this field as:
			(int (*)())routine
  int rc_mod;           Determines if parameter can be updated when
                        operator receives a "QUERY_RUNPAR" message.
			This message is usually sent during initialization
			of a run. If the value in this field is 'YES', it 
			is possible to update this variable, and if the 
			value is 'NO' that is not possible.
 ************************************************************/
  "Run number"  ,"runnr"      ,"collector",INT_DEC ,(int (*)())NULL     ,YES,
  "Max. Events" ,"maxnevents" ,"collector",INT_DEC ,(int (*)())NULL     ,YES,
  "Event source","eventsource","collector",STRING  ,(int (*)())NULL     ,YES, 
  "Event number","eventnr"    ,"collector",INT_DEC ,(int (*)())NULL     ,NO , 
  "Start Time"  ,"starttime"  ,"collector",INT_CONV,(int (*)())conv_time,NO ,
  "End Time"    ,"endtime"    ,"collector",INT_CONV,(int (*)())conv_time,NO ,
  "Status"      ,"runstate"   ,"collector",INT_CONV,(int (*)())conv_stat,NO ,
};


/**********************************************************
  The MAX_N_RP_BUT definition determines the maximum number of
  variables that can be displayed. This number should be at least equal
  to the number of parameter displays you have defined above.
 ***********************************************************/
#define MAX_N_RP_BUT 10

int n_rp_but = sizeof(rp_but)/sizeof(struct rp_def);

