/***********************************************************
 File:   commands_lib.c
 Author: C. Timmermans
 ***********************************************************
 This file contains the coding of the procedures that can be 
 called from the command.list. These routines are coded as 
 follows:
         int routine(opt)
	 char *opt;
	 {
	  **********************************
	   The string opt contains all arguments to
           this routine as given in the command.list file
	  **********************************
	  int ret_code;
	  **********************************
	   Code all actions. The return of this routine 
           can be:
           < 0 : The execution of the macro will be stopped
                 since an error occured
             0 : The routine was succesful, if this call
                 is used in an IF statement, its return is FALSE
             1 : The routine was succesful, if this call
                 is used in an IF statement, its return is TRUE
	  return(ret_code);
	 }
*********************************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include<stdio.h>
#include<strings.h>
#include<string.h>
#include<time.h>
#include<sys/time.h>
#include<unistd.h>
#include <unistd.h>
#include<sys/types.h>
#include <stdlib.h>

#include "files.h"
#include "runco.h"
#include "sdc_pack_msg_c.h"

#define  PROCESS_NAME       20      /* Process Name Length */

extern int runnr;
extern int wait_for;
extern char user_int[];
extern char ownpname[];
extern char collector[];
extern char recorder[];

char run_str[MAX_RUN_STATES][20]={STR_RUN_STATES};

int send_process(s)
char *s; 
{ 
/*********************************************************
  send_process (referred to by "ask") sends a message to a 
  process. It requires at least two parameters:
     1) The name of the process to send a message to
     2) The message to send
**********************************************************/
  char dest[50];
  char message[150];

  Parse(s,dest,message);
  Send_Message(dest,message);
  return(0);
}

int send_logbook(s) 
char *s;
{ 
/***********************************************************
 send_logbook sends a predefined message to the logbook.
 Depending on the message, it requires either one or two 
 parameters.
 ***********************************************************/
  char word[30],word2[50];
  char message[80],mess[80];

		
  Parse(s,word,message);
/********* Update the run state in the logbook *********/
  if(strcmp(word,"run_start") == 0)      send_err_string(Runco_Start);
  else if(strcmp(word,"run_end") == 0)   send_err_string(Runco_End);
  else if(strcmp(word,"run_pause") == 0) send_err_string(Runco_Suspend);
  else if(strcmp(word,"run_resume") == 0) 
    send_err_string(Runco_Resume);
  else {
/******* Report that a command is given, which is incompatible with
         the run state                         ************/
    Parse(message,word2,mess);
    add_err_string(word2);
    if(strcmp(word,"execute") == 0)       send_err_string(Runco_Execute);
    else if(strcmp(word,"incompatible") == 0)       
      send_err_string(Runco_Incompat);
    else return(-1);
  }
  return(0);
}

int update_parameters(s)
char *s;
{
/*******************************************************
 update_parameters asks new parameters from the operator.
 The arguments can be "STORAGE", in which case storage information
 is requested; or "RUN", in which case run parameters are requested
 *******************************************************/
  short sh;
  char tmp[200];
  
  if(strncasecmp(s,"RUN",3) == 0) {
    sprintf(tmp,"tell %s QUERY_RUNPAR",user_int);
    Send_Message(ownpname,tmp);
    read_value_from_sp(ownpname,"wait_for",&sh,(char *)(&wait_for));
    while((wait_for & QP_BO_RUNPAR) == 0) {
      sleep(1); 
      read_value_from_sp(ownpname,"wait_for",&sh,(char *)(&wait_for));
    }
    wait_for -= QP_BO_RUNPAR;
    sprintf(tmp,"set wait_for = %d",wait_for);
    Send_Message(ownpname,tmp);
    return(0);
  }
  if(strncasecmp(s,"STORAGE",7) == 0) {
    sprintf(tmp,"tell %s QUERY_STORAGE",user_int);
    Send_Message(ownpname,tmp);
    read_value_from_sp(ownpname,"wait_for",&sh,(char *)(&wait_for));
    while((wait_for & QP_BO_STORAGE) == 0) {
      sleep(1);
      read_value_from_sp(ownpname,"wait_for",&sh,(char *)(&wait_for));
    }
    wait_for -= QP_BO_STORAGE;
    sprintf(tmp,"set wait_for = %d",wait_for);
    Send_Message(ownpname,tmp);
    return(0);
  }
  return(-1);
}
int prompt_process(s)
char *s;
{
/********************************************************
 prompt_process requests information from a process. It
 requires two arguments:
 1) destination, i.e. the process that has to send the info
 2) message send to the process so it can identify which info to 
    send.
 *********************************************************/
	char dest[100],msg[200], rest[100];
	short sh;

	Parse(s,dest,rest);
	sprintf(msg,"prompt %s",rest);
	if(Send_Message(dest,msg) < 0) return(0);
	read_value_from_sp(ownpname,"wait_for",&sh,(char *)(&wait_for));
	while((wait_for & QP_OTHER) == 0) {
	  sleep(1);
	  read_value_from_sp(ownpname,"wait_for",&sh,(char *)(&wait_for));
	}
	wait_for -= QP_OTHER;
	sprintf(msg,"set wait_for = %d",wait_for);
	Send_Message(ownpname,msg);
	return(0);
}
int macro_exit(s)
char *s;
{
/****************************************************
 macro_exit, referenced as "exit" requires no arguments. It 
 makes sure that the execution of the macro is stopped
 *****************************************************/
  return(-1);
}
int next_runnr(s)
char *s;
{
/*********************************************************
 next_runnr updates the runnumber. It requires no arguments
 ********************************************************/
  char snd_msg[100];
  short sh;

  read_value_from_sp(collector,"runnr",&sh,(char *)(&runnr));
  if(runnr == 0) return(0);
  runnr+=1;
  sprintf(snd_msg,"SetPar runnr = %d",runnr);
  Send_Message(collector,snd_msg);
  set_value_in_sp("runnr",(char *)(&runnr));
  Send_Message(ownpname,snd_msg);
  return(0);
}

int write_begin_run(s) 
char *s;
{
/*************************************************
  write_begin_run requires no arguments. It writes some 
  begin run stats to file, and includes this file in 
  the logbook
 **************************************************/
  char tmp_buf[100],fname[100];
  int i;
  short sh;
  struct timeval tim;
  FILE *fp;

  if(decode_filename(WRITERUN,fname) < 0) {
#ifdef _DEBUG
    printf("Cannot decode the filename\n");
#endif
    return(0);
  }
  fp = fopen(fname,"w");
  if(fp == NULL) {
#ifdef _DEBUG
    printf("Cannot open the file\n");
#endif
    return(0);
  }
  fprintf(fp,"====== START OF RUN =========\n");
  read_value_from_sp(collector,"runnr",&sh,(char *)(&i));
  fprintf(fp,"RUN %d\n",i);
  read_value_from_sp(collector,"maxnevents",&sh,(char *)(&i));
  fprintf(fp,"MAXIMUM NR OF EVENTS %d\n",i);
  read_value_from_sp(collector,"starttime",&sh,(char *)(&(tim.tv_sec)));
  fprintf(fp,"START TIME %s",ctime((time_t *)&(tim.tv_sec)));
  read_value_from_sp(collector,"eventsource",&sh,tmp_buf);
  fprintf(fp,"TRIGGER TYPE %s\n",tmp_buf);
  fprintf(fp,"=============================\n");
  fclose(fp);
  sprintf(tmp_buf,"read %s",fname);
  Send_Message("logbook",tmp_buf);
  return(0);
}

int write_end_run(s)
char *s;
{
/*************************************************
  write_end_run requires no arguments. It writes some 
  end run stats to file, and includes this file in 
  the logbook
 **************************************************/
  char tmp_buf[100],fname[100];
  struct timeval tim;
  int i;
  short sh;
  FILE *fp;

  if(decode_filename(WRITERUN,fname) < 0) {
#ifdef _DEBUG
    printf("Cannot decode the filename\n");
#endif
    return(0);
  }
  fp = fopen(fname,"w");
  if(fp == NULL) {
#ifdef _DEBUG
    printf("Cannot open the file\n");
#endif
    return(0);
  }
  fprintf(fp,"====== END OF RUN =========\n");
  read_value_from_sp(collector,"runnr",&sh,(char *)(&i));
  fprintf(fp,"RUN %d\n",i);
  read_value_from_sp(collector,"eventnr",&sh,(char *)(&i));
  fprintf(fp,"TOTAL NR OF EVENTS %d\n",i);
  read_value_from_sp(collector,"maxnevents",&sh,(char *)(&i));
  fprintf(fp,"MAXIMUM NR OF EVENTS %d\n",i);
  read_value_from_sp(collector,"starttime",&sh,(char *)(&(tim.tv_sec)));
  fprintf(fp,"START TIME %s",ctime((time_t *)&(tim.tv_sec)));
  read_value_from_sp(collector,"endtime",&sh,(char *)(&(tim.tv_sec)));
  fprintf(fp,"END TIME %s",ctime((time_t *)&(tim.tv_sec)));
  read_value_from_sp(collector,"eventsource",&sh,tmp_buf);
  fprintf(fp,"TRIGGER TYPE %s\n",tmp_buf);
  fprintf(fp,"=============================\n");
  fclose(fp);
  sprintf(tmp_buf,"read %s",fname);
  Send_Message("logbook",tmp_buf);
  return(0);
}

int check_run_status(s)
char *s;
{
/*************************************************************
 check_run_status checks if the actual run status corresponds
 with the status given in the argument. It requires one 
 argument corresponding to the asumed runstate
 *************************************************************/
  int i;
  short sh;
  char word[30];

  sscanf(s,"%s",word);
  read_value_from_sp(collector,"runstate",&sh,(char *)(&i));
  if(strcmp(word,run_str[i]) == 0) return(1);
  return(0);
}



