/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      July 27, 1992                                            */
/*        rask.c                                                 */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <stdio.h>

#include <sys/time.h>

#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "rask.h"
#include "template.h" 

int Process_Event();
extern char *get_name_from_args(); 
void collate_strings();
/*
 *   For NOVA
 */

void interrupt_handler();

/*(a+out) ------------------------------------------------- */
main(argc, argv)
int argc;
char *argv[];
/*(b+out) -------------------------------------------------
Procedure Name
Main - 
Synopsis
Description
main() is the main procedure of the histogram. First it creates
another analyzer process which is responsible for the paw interface.
Next, it opens the buffer manger. It then calls the template with 
the addresses of the Process_Event() and display_macro procedures.  

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */                             
{
   int process_ID;
   char args[256], p_loc[50], pwho[50];
   Table entry;
   int is_n = 0;
   int argcnt = 0;
   int temp;
   struct tm *now;
   time_t nowtime;
   char timstr[10];

/*         Parse out the command line options              */
   strcpy(p_loc,get_name_from_args(argc,argv));
   collate_strings(argc,argv,args,&is_n,&argcnt,pwho);
   if (debug_level >= 20)  {
     printf (" Collated arg string (%d arg) is %s\n", argcnt, args);
     printf (" Messages are to be sent to %s\n", pwho);
   }
/*         construct a unique process_name, if necessary   */
   if (is_n == 0)  {
     if (Attach_Table() != -1) {
      temp = Search_Table(p_loc);
      if (temp >= 0) {
         Read_from_Table(&entry,temp);
         if (entry.status != DEAD) {
           nowtime = time(NULL);
           now = localtime(&nowtime);
           sprintf(timstr, "%02d%02d", now->tm_sec,now->tm_min);
           temp = strlen(p_loc);
           strcat(p_loc,timstr);
           p_loc[temp+4] = '\0';
           if (debug_level >= 20) printf (" Modify process name to %s\n",
		    p_loc);
	 }
      }
      Detach_Table();
    }
   }

   process_ID = fork();

   if(process_ID < 0) {
     printf("Cannot fork a child process\n");
     exit(1);
   }
   else if (process_ID > 0) {              /*  This is the parent  */
      if (argcnt == 0)  {
	printf ("Which process? ");
	gets(pwho);
      }
      strcpy(ownpname,p_loc);
      user_interface(pwho,argcnt,args);
   }
   else {                                  /*  This is the child   */
      if(Template_Init(p_loc) < 0) {
	printf("Process of name %s already exists in unidaq\n",
	       get_name_from_args(argc,argv));
	printf("Please choose a different name using -n\n");
	exit(0);
      }

      Template_Main((int (*)()) Process_Event, NULL, NULL);
   }
}


void collate_strings(argc, argv, args, is_n, argcnt, pwho)
int argc;
char **argv;
char *args, *pwho;
int  *is_n, *argcnt;
/*   Pick out the -n and -m options from the command line and catenate
 *   all the other strings together, blank separated.                 
 */
{
   int k;
   *is_n = 0;
   args[0] = '\0';
   *argcnt = 0;

   if (argc != 0) {
     for (k=1; k<argc; k++)  {
       if      (strcmp(argv[k],"-n") == 0)  {
	        k++;
		*is_n = 1;
       }
       else if (strcmp(argv[k],"-m") == 0)  {
	        k++;
       }
       else    {
	        strcat(args,argv[k]);
		strcat(args," ");
		*argcnt += 1;
		if (*argcnt == 1) strcpy(pwho,argv[k]);
       }
     }
   }
}




/*(a+out) ------------------------------------------------- */
int Process_Event()
/*(b+out) -------------------------------------------------
Procedure Name
Process_Event - processes an event
Synopsis
Description
Process_Event() processes an event.       

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   return(TRUE);
}




/*(a+out) ------------------------------------------------- */
int ExitA(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
ExitA - processes a display command
Synopsis
Description
Display(header, arg_string) processes a display command.     
header: the name of the process which sends the display cmnd,
arg_string: the arguments of the display command.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   kill(getppid(), SIGINT);
   Exit(ownpname, " ");
}



/*(a+out) ------------------------------------------------- */
user_interface(prompt,nargs,args)
char *prompt,*args;
int nargs;
/*(b+out) -------------------------------------------------
Procedure Name
User_interface - interacts with the user for PAW cmnds.
Synopsis
Description
user_interface(semid) interacts with the user for PAW cmnds.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   char msg[80], string[102], pname2[20];
   int flag, index;
   int j;
   char header[20];
   Table entry;
   struct sigvec vec;
 
   sleep(3); /* Wait until child is initialized. */

#ifdef SGI
   vec.sv_handler = (int (*)()) interrupt_handler;
#else
   vec.sv_handler = (void (*)()) interrupt_handler;
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(SIGINT, &vec, (struct sigvec *) NULL);
#else
   sigvec(SIGINT, &vec, (struct sigvec *) NULL);
#endif

   index = -1;
   j = 0;
   strcpy(pname2, ownpname);
   while ((index==-1) && (j<5) )  {
     if (Attach_Table() == -1) printf ("rask: Error in Attach_Table()\n");
     index = Search_Table(pname2);
     if (index == -1) {
        printf("rask: Error finding self (%s) in Search_Table()\n", pname2);
	if (j==4) exit (1);
	Detach_Table();
	sleep(2);
     }
     j++;
   }
 
   if (nargs > 1) {
     Parse (args, header, string);
     Send_Message(prompt,string);
     sleep(1);
     Send_Message(pname2,"EXIT");
     sleep(10);
   }
   else  {
      flag = TRUE;
      while (flag == TRUE) {
         printf ("%s > ", prompt);
         gets(msg);
           if (strncasecmp(msg, "STOP",
                     (strlen(msg) > 1) ? strlen(msg) : 2)  == 0) {
              flag = FALSE;
	      Send_Message(pname2,"EXIT");
	      sleep(10);
           }
           else if (strncasecmp(msg, "QUIT",
                     (strlen(msg) > 1) ? strlen(msg) : 2)  == 0) {
              flag = FALSE;
	      Send_Message(pname2,"EXIT");
	      sleep(10);
           }
           else {
	     Send_Message(prompt,msg);
             if (strncasecmp(msg, "EXIT",
                     (strlen(msg) > 1) ? strlen(msg) : 2)  == 0) {
               flag = FALSE;
	       Send_Message(pname2,"EXIT");
	       sleep(10);
             }
	     else {
               sleep(2);
	     }
	   }
      }
   }
}




/*(a+out) ------------------------------------------------- */
void interrupt_handler()
/*(b+out) -------------------------------------------------
Procedure Name
interrupt_handler - 
Synopsis
Description
interrupt_handler 

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   exit(0);
}

