/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      July 27, 1992                                            */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifndef NODYN      /*  Ensure Non-dynamic or Dynamic compilation is set  */
#ifndef YESDYN
#define YESDYN
#endif
#endif

#include <stdio.h>
#include  <setjmp.h>
      jmp_buf env;

#ifdef YESDYN           /*  Dynamic  */
#ifdef SUN 
#include <dlfcn.h>
#endif

#ifdef HP 
#include <dl.h>
#endif
#endif                 /*  End Dynamic  */

#include <sys/time.h>

#ifdef SGI
#define _BSD_SIGNALS
#ifdef YESDYN          /*  More Dynamic  */
#include <dl.h>
#include <ar.h>
#include <filehdr.h>
#include <syms.h>
#include <ldfcn.h>
#endif                /*  End More Dynamic  */
#endif

#include <signal.h>
#include <string.h>
#include "analyzer.h"
#include "template.h" 

#ifdef NODYN
#include "NDanalyzer.h"
#endif

int Process_Event();
int display_macro();
void setfpname_();
char *get_name_from_args();

#ifdef YESDYN
void Dlt_allp();
struct files *srchfile();
struct procedures *srchprc(); 
#endif

extern paw_init_();
extern paw_exec_();
extern paw_exit_();
int    decode_filename(); 
/*
 *   For NOVA
 */
#include "nova.h"

#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */
#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      20
#define DEFAULT_SLEEP    1

#define  NTIMEOUT        0

void interrupt_handler();
void special_handler(); /* longjump to command input routine */

struct   NOVAD *nd;

#ifdef YESDYN
/*
 *   For Dynamic Linking
 */
 
struct procedures {
   char procname[20];
   int  (*funcptr)();
   struct procedures *next;
};

 struct files {
   char filename[20];
#ifdef SUN
   int *file_handle;
#endif
#ifdef HP
   shl_t file_handle;
#endif
#ifdef SGI
   LDFILE *file_handle;
#endif
   struct procedures *first_proc;
	 struct files *next;
};

static struct files *first_file = NULL;

#endif                   /* end, for dynamic loading */

/*
 *   For Display Macro
 */

struct {
   char macroname[20];
   int  interval;
   int  lastdisplay;
   int  on;
} macro;


int process_ID;
int semid;

#ifdef SGI
   int f77argc;                /* allows linkin in fortran */
   char **f77argv;
   struct nlist namels[10];
#endif


/*
 *   For history of analyzer commands
 */
#define HISTORY		20	/* number of commands remembered */	
void copy_history();


/*(a+out) ------------------------------------------------- */
main_entry(argc, argv)
int argc;
char *argv[];
/*(b+out) -------------------------------------------------
Procedure Name
Main_entry - 
Synopsis
Description
main_entry() is the main procedure of the histogram. First it creates
another analyzer process which is responsible for the paw interface.
Next, it opens the buffer manger. It then calls the template with 
the addresses of the Process_Event() and display_macro procedures.  

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */                             
{
   struct sigvec vec;
   char pname1[20];
   char name[NV_MAXNAME + 1]; 
   char sec_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];
   
#ifdef SGI
   for_rtl_init_(&argc, argv);      /* allows link in fortran */
#endif

   strcpy(pname1,"                   ");
   macro.on = FALSE;

   if(Create_Sem(&semid)==ERROR) {
	printf("Analyzer exiting, cannot create semaphore\n");
	printf("Use Reset to remove semaphores \n");
	exit(1);
   }

   process_ID = fork();

   if(process_ID < 0) {
     printf("Cannot fork a child process\n");
     exit(1);
   }
   else if (process_ID > 0) {

      paw_init_();

      if(Template_Init(get_name_from_args(argc,argv)) < 0) {
	printf("Process of name %s already exists in unidaq\n",
	       get_name_from_args(argc,argv));
	printf("Please choose a different name using -n\n");
	exit(0);
      }

/*
 *  Set the process name in a fortran common block for use by 'ana_command'
 */
      strcpy(pname1,ownpname);
      pname1[strlen(ownpname)] = ' ';
      setfpname_(pname1);

      UNI_nova_parms(DEFAULT_NAME, DEFAULT_PRI, name, sec_name);
      if ((int)(nd = (struct NOVAD *)UNI_nova_open(name, nova_prio, 1)) 
                   == -1) {
          printf("Error in opening nova...\n");
          if (debug_level >= 0) printf("cannot open %s", name);
          Send_Message ( ownpname, "EXIT" );
      }
      Update_Sem_Wait(semid, 1); /* let user_interface know we are ready */

      setjmp(env); /* this is where then interrupt handler will jump to */


/* install  signal handler for control-C after the default KUIP one */
#ifdef SGI
   vec.sv_handler = (int (*)()) special_handler;
#else
   vec.sv_handler = (void (*)()) special_handler;
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(SIGINT, &vec, (struct sigvec *) NULL);
#else
   sigvec(SIGINT, &vec, (struct sigvec *) NULL);
#endif

      Template_Main((int (*)()) Process_Event, NULL, 
                    (int (*)()) display_macro);
   
   }
   else { /* user interface is the child */
        strcpy(ownpname,get_name_from_args(argc,argv));
         user_interface(semid);
   }
}





/*(a+out) ------------------------------------------------- */
int Process_Event()
/*(b+out) -------------------------------------------------
Procedure Name
Process_Event - processes an event
Synopsis
Description
Process_Event() processes an event.       

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   struct files *f;
   struct procedures *p;
  /* struct timeval tp;
   struct timezone tzp; */
   char  num[20];
   int    status, pos;
   int 	*data;
   int length;

   /* 
    *   Check for event
    */
   /* nova get : keep trying to get a buffer */

   status = UNI_nova_get(nd, &data, &length);
   if (status != 0) return(FALSE);
   pos = 0;
   while (pos*4 < length) {
      if (data[pos] == 0) break;
      if (debug_level > 10) {
         if (data[pos + 1] == EVENT_RECORD)
         if (debug_level > 20) printf("Event buffer received.\n");
         else if (data[pos + 1] == BEGINRUN_RECORD)
            printf("BeginRun record received.\n");
         else if (data[pos + 1] == PAUSERUN_RECORD)
            printf("PauseRun record received.\n");
         else if (data[pos + 1] == RESUMERUN_RECORD)
            printf("ResumeRun record received.\n");
         else if (data[pos + 1] == ENDRUN_RECORD)
            printf("EndRun record received.\n");
      }

      if (data[pos + 1] == BEGINRUN_RECORD) {
         sprintf(num,"%d", data[pos+2]);
         Begin(ownpname, num);
         pos += data[pos];
      }
      else {
         if (data[pos + 2] != run_number) {
            sprintf(num,"%d", data[pos+2]);
            printf(" Warning: unexpected change of runnumber: ");
	    printf(" %d --> %d\n", run_number, data[pos+2]);
            Begin(ownpname, num);
         }
         if (data[pos + 1] == ENDRUN_RECORD) {
            End();
         }
         else if (data[pos + 1] == EVENT_RECORD) {
         }
#ifdef NODYN
	 do_event_ND(&data[pos]);
#else
         do_event(&data[pos]);
#endif
         pos += data[pos];
      }
   };

   UNI_nova_put(nd, data, length, 0);
   return(TRUE);
}

#ifdef YESDYN
do_event(data)
int *data;
{    struct files *f;
     struct procedures *p;

            f = first_file;
            while (f != NULL) {
               p = f->first_proc;
               while (p != NULL) {
                  p->funcptr(&data[6],&data[0]);
                  p = p->next;
               }
               f = f->next;
            }
return;
}
#endif

#ifdef NODYN
do_event_ND(data)
int *data;
{ 
  struct void_function_struct *PTvf=nd_func_list;

  while (PTvf->PTfunc) 
    {
      PTvf->PTfunc(&data[6],&data[0]);
      PTvf++;
    }

  return;
}
#endif



/*(a+out) ------------------------------------------------- */
int Begin(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Begin - processes a begin command
Synopsis
Description
Begin(header, arg_string) processes a begin command.  It also 
execute the andbegin.kumac macro. 
header: the name of the process which sends the begin cmnd,  
arg_string: the arguments of the begin command. 

Superconduction Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   char command[100];

   run_number = atoi(arg_string);
   runstate = BEGIN_STATE;
   strcpy(command,"exec anbegin.kumac runnumber=");
   strcat(command,arg_string);
   Paw(ownpname, command);
   printf("Begin Run # %d \n",run_number);

   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int End(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
End - Main procedure of histogram
Synopsis
Description
End(header, arg_string) processes an end command. It also
executes the anend.kumac macro.           
header: the name of the process which sends the end command, 
arg_string: the arguments of the end command.  	 		     

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */   

{
   if (debug_level >= 20) printf("End()...\n");
   runstate = END_STATE;
   Paw(ownpname, "exec anend.kumac  ");
   return TRUE;
}

 
/*(a+out) ------------------------------------------------- */
int ComProc(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
ComProc - Execute template commands from a named file
Synopsis
Description
ComProc(header, arg_string) processes a ComProc command, to
execute template commands from a named file.  
header: the name of the process which sends the end command, 
arg_string: the name of the file.  It will be parsed for< a
            leading environmental variable.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */   

{
  FILE *fp;
  char line[80], fnam[80], big_line[80], temp1[80], temp2[80];
  int j;

  Parse(arg_string,fnam,temp1);
  if ( ((j=decode_filename(fnam,arg_string)) != -2) && (j!=0)) {
    printf ("Cannot understand filename '%s' to '%s'\n", fnam, arg_string);
    return (TRUE);
  }
  if (j==-2) strcpy(arg_string,fnam);

  if ( (fp = fopen(arg_string,"r")) != NULL)  {
    while ((int)fgets(line,80,fp) > 0)  {
      line[strlen(line)-1] = '\0';
      printf ("===> %s\n", line);
/*                Do not allow this routine to be re-entered      */
      sscanf(line,"%s",temp1);
      if ((strncasecmp("COMPROC",temp1,strlen(temp1)) == 0) ||
	   (temp1[0] == '@') ){
	printf("        Sorry, ComProc is not re-entrant\n");
      }
/*                Put the pawc string on the head, if needed,
 *                and process the command                         */
      else  {
        Add_Header(line, big_line);
	if ((Parse_Command_Usr_Part(big_line, temp1, temp2) < 0) &&
	    (Parse_Command_Sys_Part(big_line, temp1, temp2) < 0))   {
	  strcpy(big_line,ownpname);
	  strcat(big_line," pawc ");
	  strcat(big_line,line);
	}
	if (Process_Command(big_line,NULL) != TRUE) {
	  printf("        Bad command status!!!  Abort processing.\n\n");
	  break;
	}
      }
    }
    fclose(fp);
  }
  else {
    printf ("\nCommand file %s not found\n", arg_string);
  }
  return (TRUE);
}



/*(a+out) ------------------------------------------------- */
int Paw(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Paw - Main procedure of histogram
Synopsis
Description
Paw(header, arg_string) processes a PAW command.             
header: the name of the process which sends the PAW cmnd,   
arg_string: the arguments of the PAW command.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   char interface[102];
   int i;
   
   strcpy(interface, arg_string);
   for (i=strlen(arg_string); i < 101; i++)
      interface[i] = ' ';
   interface[101] = '\0';
   paw_exec_(interface);
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int PawC(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
PawC - processes a PAW command.
Synopsis
Description
PawC(header, arg_string) processes a PAW command.            
header: the name of the process which sends the PAW cmnd,  
arg_string: arguments of the PAW command.     		                  	                              								
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */                   
{
   char interface[102];
   int i;
 
   strcpy(interface, arg_string);
   for (i=strlen(arg_string); i < 101; i++)
      interface[i] = ' ';
   interface[101] = '\0';

   Update_Sem_Wait(semid, -1);
      paw_exec_(interface);
   Update_Sem_Wait(semid, 1);
   return TRUE;
}

 

/*(a+out) ------------------------------------------------- */
int Display(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Display - processes a display command
Synopsis
Description
Display(header, arg_string) processes a display command which
is responible for reading in the name of the macro which is
to be executed.    
header: the name of the process which sends the display cmnd,
arg_string: the arguments of the display command (which include
the name of the macro, and the interval between execution of
the macro.)

Examples:
.sp .5
\s-1 display [macro] [time in sec.]
\s-1 display macro1 5
\s-1 display off
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   char   macroname[20], 
          temp1[100], 
          temp2[100], 
          interval[5], 
          modifier[20],   
          macro_cmd[102];
   int    i, time_interval;
   struct timeval tp;
   struct timezone tzp;

 
   Parse(arg_string, modifier, temp1);
   
   /*
    *   Display off command.
    */
   if (strcasecmp(modifier, "off") == 0)  { 
       macro.on = FALSE;
       return TRUE;
   }
   
   /*
    *   Display command 
    */
   strcpy(macroname, modifier);
   Parse(temp1, interval, temp2);
   
   if ((time_interval = atoi(interval)) == 0) 
      time_interval = 10;

   strcpy(macro.macroname, macroname);
   macro.interval = time_interval;
   gettimeofday(&tp, &tzp);
   macro.lastdisplay = tp.tv_sec;
   macro.on = TRUE;

   /*
    *   Display macro
    */ 
   strcpy(macro_cmd, "exec ");
   strcat(macro_cmd, macro.macroname);
   strcat(macro_cmd, " FIRST=YES  ");
   for (i=strlen(macro_cmd)-1; i < 101; i++) macro_cmd[i] = ' ';
   macro_cmd[101] = '\0';
   paw_exec_(macro_cmd);

   return TRUE;
}
 

#ifdef YESDYN        /*  Start of Dynamic link routines  LINK,  */
                     /*     UNLINK, ADD, DELETE.                */


/*(a+out) ------------------------------------------------- */
int Link(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Link - processes a dynamic link command
Synopsis
Description
Link(header, arg_string) processes a dynamic link command.   
header: the name of the process which sends the PAW cmnd,    
arg_string: the arguments of the link command.

Examples:
.sp .5
\s-1 link [filename]
\s-1 link histogram1.f

Superconduction Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   FILE * file_check;
   struct files *f;
   char   filename[255], 
          temp1[100], 
          command[100], 
          command2[100], 
	  errorcode[100],
          temp[100],
          temp2[100];
   int    lngth;
   int    rv ;
#ifdef SUN
   int    *file_handle;
   int    bk_mask;
#endif
#ifdef HP
   shl_t file_handle;
   long o_msk;
#endif
#ifdef SGI
   LDFILE  *file_handle;
   int bk_mask;
#endif   
   
#if defined(SUN) || defined(HP) || defined(SGI)  
   
   int i, MARK;
  /*
   * Check to see if makefile is in the directory
   */
  if((file_check = fopen("analyzermake", "r")) != NULL)
      fclose(file_check);
   else {
      printf(" Creating a makefile called 'analyzermake' \n");
      printf(" You may wish to customize this file \n");
      strcpy(command,"cp $USERUNIDAQ/anal/analyzermake .");
      system(command);
      sleep(2);
   } 

 
   Parse2(arg_string, filename, temp1);
do {
   /*
    * check to see if the file is already linked.
    */
   f = srchfile(filename, first_file); 
   
   if (f != NULL) {
      if (debug_level >= 0) printf("File is linked already.\n");
      return TRUE;
   }

   lngth = strlen(filename);

   if (filename[lngth-1] == 'c') {
      strcpy(command,"IGNORE ON ");
      strcat(command,ownpname);
      Send_Message("xpc", command);

#ifdef HP
      o_msk = sigsetmask(0xFFFFFFFF);
#endif
#ifdef SGI
      bk_mask = sigsetmask(0xFFFF);
#endif
#ifdef SUN
      bk_mask = sigsetmask(0xFFFFFFFF);
#endif
      printf("Linking a C program.\n");
      strcpy(command, "make -f analyzermake \"AMAKEWHCH=");
      strncpy(temp, filename, lngth-2);
      temp[lngth-2] = '\0';
      strcat(command, temp);
      strcat(command, "\" ");
      strcat(command, temp);
      printf("command = %s \n",command);
      rv = system(command);
#ifdef HP
      sigsetmask(o_msk);
#endif
#ifdef SGI
      sigsetmask(bk_mask);
#endif
#ifdef SUN
      sigsetmask(bk_mask);
#endif
      strcpy(command,"IGNORE OFF ");
      strcat(command,ownpname);
      Send_Message("xpc", command);

      if (rv) {
	if (debug_level >= 0) printf("Make error!\nFile not linked.\n");
	return TRUE;
	}
#ifdef SUN
      file_handle = dlopen(temp, 1);
      if (dlerror() != (char *) NULL) {
         if (debug_level >= 0) printf("Error in dlopen()...\n");
         return TRUE; 
      }
#endif
#ifdef HP
      if((file_handle = shl_load(temp,BIND_DEFERRED,0)) == NULL) {
      return TRUE;
      }
#endif
#ifdef SGI
/* Just to keep a checking here */
      {
   FILE *tmpfp;
      strcat (temp, ".o");
      printf ("temp = %s\n", temp);
      tmpfp = fopen(temp, "r");
      if (tmpfp == NULL) {
	  if (debug_level >= 0) 
		printf("Error in opening %s ...\nFile is not linked.\n", temp);
	  return TRUE;
	  }
      else fclose(tmpfp);
      if((file_handle = ldopen(temp,NULL)) == NULL) {
	  if (debug_level >= 0) 
		printf("Error in opening %s ...\nFile is not linked.\n", temp);
	  return TRUE;
	  }
      }
#endif

   }

   else if(filename[lngth - 1] == 'f') {
      printf("Linking a FORTRAN program.\n");
      strcpy(command,"IGNORE ON ");
      strcat(command,ownpname);
      Send_Message("xpc", command);

#ifdef SUN
      strcpy(command, "make -f analyzermake \"MACHINE=SUN\" \"AMAKEWHCH=");
      bk_mask = sigsetmask(0xFFFFFFFF);
#endif
#ifdef HP
      o_msk = sigsetmask(0xFFFFFFFF);
      strcpy(command, "make -f analyzermake \"MACHINE=HP\" \"AMAKEWHCH=");
#endif
#ifdef SGI
      bk_mask = sigsetmask(0xFFFF);
      strcpy(command, "make -f analyzermake \"MACHINE=SGI\" \"AMAKEWHCH=");
#endif

      strncpy(temp, filename, lngth-2);
      temp[lngth-2] = '\0';
      strcat(command, temp);
      strcat(command, "\" ");
      strcat(command, temp);
      printf("command = %s \n",command);
      rv = system(command);
#ifdef HP
      sigsetmask(o_msk);
#endif
#ifdef SGI
      sigsetmask(bk_mask);
#endif
#ifdef SUN
      sigsetmask(bk_mask);
#endif     
      strcpy(command,"IGNORE OFF ");
      strcat(command,ownpname);
      Send_Message("xpc", command);

      if (rv) {
	if (debug_level >= 0) printf("Make error!\nFile not linked.\n");
	return TRUE;
	}
#ifdef SUN
      file_handle = dlopen(temp, 1);
      if (dlerror() != (char *) NULL) {
         if (debug_level >= 0) printf("Error in dlopen()...\n");
         return TRUE;
      } 
#endif
#ifdef HP
      if((file_handle = shl_load(temp,BIND_DEFERRED,0)) == NULL) {
      return TRUE;
      } 
#endif
#ifdef SGI
/* Just to keep a checking here */
      {
   FILE *tmpfp;
      strcat (temp, ".o");
      printf ("temp = %s\n", temp);
      tmpfp = fopen(temp, "r");
      if (tmpfp == NULL) {
	  if (debug_level >= 0) 
		printf("Error in opening %s ...\nFile is not linked.\n", temp);
	  return TRUE;
	  }
      else fclose(tmpfp);
      if((file_handle = ldopen(temp,NULL)) == NULL) {
	  if (debug_level >= 0) 
		printf("Error in opening %s ...\nFile is not linked.\n", temp);
	  return TRUE;
	  }
      }
#endif

    }
   else {                     /* just load it */
       strcpy(temp,filename);
#ifdef SUN
      file_handle = dlopen(temp, 1);
      if (dlerror() != (char *) NULL) {
         if (debug_level >= 0) printf("Error in dlopen()...\n");
         return TRUE;
      }
#endif
#ifdef HP
      if((file_handle = shl_load(temp,BIND_DEFERRED,0)) == NULL) {
      return TRUE;
      }
#endif
#ifdef SGI
/* Just to keep a checking here */
      {
   FILE *tmpfp;
      strcat (temp, ".o");
      printf ("temp = %s\n", temp);
      tmpfp = fopen(temp, "r");
      if (tmpfp == NULL) {
	  if (debug_level >= 0) 
		printf("Error in opening %s ...\nFile is not linked.\n", temp);
	  return TRUE;
	  }
      else fclose(tmpfp);
      if((file_handle = ldopen(temp,NULL)) == NULL) {
	  if (debug_level >= 0) 
		printf("Error in opening %s ...\nFile is not linked.\n", temp);
	  return TRUE;
	  }
      }
#endif

    }

   /*
    *   Add new file to the files list
    */
   f = (struct files *) malloc(sizeof(struct files));
   strcpy(f->filename,filename);
   f->file_handle = file_handle;
   f->next = first_file;
   f->first_proc = NULL;
   first_file = f;
   
   if (debug_level >= 0) printf("Link completed: %s\n",f->filename); 

   /*
    * check to see if the user included the functions to be
    * add enclosed in parenthesis. ie send to ADD function. 
    * example: link histogram1.f (fill_hist1 fill_hist2 )
    */
   i = -1;
   MARK = TRUE;
   do {
      i++;
      if(temp1[i] == '(') {
         strcpy(command2,filename);
         strcat(command2," ");
         while(temp1[i] != ')') {
            i++; 
            if(temp1[i] != ')') strncat(command2,temp1+i,1); }
         Add(ownpname, command2);
         i++;
         strcpy(temp2,temp1+i);
         strcpy(temp1,temp2);
         MARK = FALSE;
      }
      /* found a file */
      if((temp1[i] >= 'a' && temp1[i] <= 'z') ||
         (temp1[i] >= 'A' && temp1[i] <= 'Z')) {
        MARK = FALSE;
      }
   }while(MARK); 

   Parse2(temp1, filename, temp1);

/* check to see if command enclosed more than one program to link */

}while(strlen(filename)> 1);
   return TRUE;
#endif
}



/*(a+out) ------------------------------------------------- */
int Unlink(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Unlink - processes an unlink command.
Synopsis
Description
Unlink(header, arg_string) processes an unlink command.
header: the name of the process which sends the PAW cmnd,
arg_string: the arguments of the link command.

Examples:
.sp .5
\s-1 unlink [filename]
\s-1 unlink histogram1.f
\s-1 unlink all     

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   struct files *f, 
          *before;
   char filename[30], 
        temp1[30];
   
#if defined(SUN) || defined(HP) || defined(SGI)
   Parse(arg_string, filename, temp1);
   /*
    * Unlink all procedures and close all files
    */
   if (debug_level >= 20) printf("the input file is= %s\n",filename); 
   if (strcasecmp(filename,"all") == 0) {
      while (first_file != NULL) {
         Dlt_allp(first_file);
         f = first_file;
         first_file = f->next;
#ifdef SUN
         if (dlclose(f->file_handle) != 0) {
#endif
#ifdef HP
	if (shl_unload(f->file_handle) != 0) {
#endif
#ifdef SGI
/* Just to keep a nicer look */
	if (ldclose(f->file_handle) != 1) {
#endif
            if (debug_level >= 0) printf("Error in dlclose()...\n");
               return TRUE;
         }
         free(f);
      }
      if (debug_level >= 0) printf("Unlink all files.\n");
      return TRUE;
   }
   /*
    *   find file
    */
      
   f = srchfile(filename,first_file);
   if (f == NULL) {
      if (debug_level >= 0) printf("File not found.\n");
      return TRUE;
   }
   if (f != first_file) {
      before = first_file;
      while (before->next != f) before = before->next;
      before->next = f->next;
   }
   else {
      first_file = f->next;
   }
   Dlt_allp(f);

#ifdef SUN
   if (dlclose(f->file_handle) != 0) {
#endif
#ifdef HP
   if (shl_unload(f->file_handle) != 0) {
#endif
#ifdef SGI
/* Just to keep a nicer look */
   if (ldclose(f->file_handle) != 1) {
#endif
      if (debug_level >= 0) printf("Error in dlclose()...\n");
         return TRUE;
   }
   free(f);
   if (debug_level >= 0) printf("Unlink %s file.\n",filename);

#else
	printf("Unlink -- is not implemented in this machine.\n");
#endif
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int Add(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Add - processes a use command.
Synopsis
Description
Add(header, arg_string) processes an add command.
header: the name of the process which sends the PAW cmnd,
arg_string: the arguments of the link command. 

Examples:
.sp .5
\s-1 add [filename] [entry1] .... [entryn]
\s-1 add histogram1.f fill_hist1 fill_hist2    

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   struct procedures *p;
   struct files *f;
   char filename[30], 
        procname[30],
        command[100],
        temp1[100],
        temp2[100];
#ifdef SUN
   int *fill_procaddr;
#endif
#ifdef HP
   long fill_procaddr;
#endif
#ifdef SGI
    dl_funcptr fill_procaddr;
    char   objname[80];
    int num_proc=0, r_num_proc, i;
    int bk_mask;
#endif

#if defined(SUN) || defined(HP) || defined(SGI) 
   Parse(arg_string, filename, temp1);

   /*
    *   Search for file
    */
   f = srchfile(filename, first_file);
   if (f == NULL) {
      if (debug_level >=0) printf("Error: No such file...\n");
      return TRUE;
   }
   /*
    *   Adding entries to the list
    */
   do {
      Parse(temp1, procname, temp2);
      strcat(procname,"_");
      if (debug_level >= 20) 
         printf("filename= %s, procname= %s\n", filename, procname);
      if (strcmp(procname, "_") != 0) {
         p = srchprc(procname, f->first_proc);
         if (p == NULL) {
#ifdef SUN
            fill_procaddr =  dlsym(f->file_handle, procname);
            if (fill_procaddr == (void *) NULL) {
#endif
#ifdef HP
	    if( shl_findsym(&f->file_handle,procname,TYPE_PROCEDURE
			    ,&fill_procaddr) == -1) {
#endif
#ifdef SGI
	    {   int lngth;
		if (p == NULL) {
		   lngth = strlen(filename);
		   strncpy(objname, filename, lngth-2);
		   objname[lngth-2] = '\0';
		   strcat (objname, ".o");
		   namels[num_proc].n_name = (char *)malloc(strlen(procname)+1);
		   strcpy (namels[num_proc].n_name, procname);
		   num_proc++;
		    }
		else {
		    if (debug_level >= 0) printf("File already added\n");
			}
		}
	    if (0) {
#endif
               if (debug_level >= 0) printf("Cannot find %s\n", procname);
            }
            else {
#ifndef SGI
               p = (struct procedures *) malloc(sizeof(struct procedures)); 
               strcpy(p->procname, procname);
               p->funcptr = (int (*) ()) fill_procaddr;
               p->next = f->first_proc;
               f->first_proc = p;
#endif
            }
         }
         else {
            if (debug_level >= 0) printf("File already added\n");
         }
      }
      strcpy(temp1, temp2);
   } while (strcmp(temp1,"\0") != 0);
#ifdef SGI
    strcpy(command,"IGNORE ON ");
    strcat(command,ownpname);
    Send_Message("xpc", command);

    bk_mask = sigsetmask(0xFFFF);
    Setval_Sem(semid,0);
	for (i=0; i < num_proc; i++) 
printf (" symbol: %s\n", namels[i].n_name);

	r_num_proc = dl_loadmod_mult ("analyzer", objname, namels);
printf (" num of symbols searched %d\n", r_num_proc);
	for (i=0; i< r_num_proc; i++) {
           p = srchprc(namels[i].n_name, f->first_proc);
	   if (namels[i].n_type) {
                p = (struct procedures *) malloc(sizeof(struct procedures)); 
                strcpy(p->procname, procname);
		fill_procaddr = namels[i].n_value;
                p->funcptr = (int (*) ()) fill_procaddr;
                p->next = f->first_proc;
                f->first_proc = p;
		}
            else if (debug_level >= 0) printf("Cannot find %s\n", namels[i].n_name);
	    }
    Setval_Sem(semid,1);
    sigsetmask(bk_mask);
    strcpy(command,"IGNORE OFF ");
    strcat(command,ownpname);
    Send_Message("xpc", command);

#endif
   
#else
	printf("ADD -- is not implemented in this machine.\n");
#endif
  return TRUE;         
} 





/*(a+out) ------------------------------------------------- */
int Delete(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Delete - processes an delete command.
Synopsis
Description
Unlink(header, arg_string) processes an delete command.
header: the name of the process which sends the PAW cmnd,
arg_string: the arguments of the link command. 

Examples:
.sp .5
\s-1 delete [filename] [entry1] .... [entryn]
\s-1 delete histogram1.f fill_hist1 fill_hist2
\s-1 delete histogram1.f all
\s-1 delete all    

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   struct files *f;
   struct procedures *p, 
                     *before;
   char procname[30], 
        filename[30],
        temp1[100], 
        temp2[100]; 
  
   Parse(arg_string, filename, temp1);
   /*
    *  For Delete all
    */
   if (strcasecmp(filename, "all") == 0) {
      f= first_file;
      while (f != NULL) {
         Dlt_allp(f);
         f=f->next;
      }
      return TRUE;
   }
   
   f = srchfile(filename,first_file);
   if (f == NULL) {
      if (debug_level >=0) printf("Error: No such file...\n");
      return TRUE;
   }
   /*
    *  Delete [filename] all command
    */  
   Parse(temp1, procname, temp2);
   if (strcasecmp(procname, "all") == 0) {
     Dlt_allp(f);
   }
   /*
    *  Delete [filename] [entry]...[entry n] command
    */  
   else {
      strcat(procname, "_");
      p= f->first_proc;
      while (strcmp(procname, "_") != 0 ) {
         while (p != NULL) {
            if (strcmp(procname, p->procname) == 0) {
               if (p == f->first_proc) {
                  f->first_proc = p->next;
                  free(p);
                  p = f->first_proc;
               }
               else {
                   before->next = p->next;
                   free(p);
                   p = before->next;
               }
               if (debug_level >= 20) printf("Delete entry: %s..\n", procname);
            }
            else {
               before = p;
               p = before->next;
            }
         };
         p = f->first_proc;
         strcpy(temp1, temp2);
         Parse(temp1, procname, temp2);
         strcat(procname, "_");     
      };
  }
   
   return TRUE;
   
}
    

#endif               /*  End of Dynamic link routines  LINK,  */
                     /*     UNLINK, ADD, DELETE.              */



/*(a+out) ------------------------------------------------- */
int ExitA(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
ExitA - Synopsis
Description
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
#ifdef YESDYN
   Unlink(ownpname,"all");
#endif
   Delete_Sem(semid); /* need to cleanup semaphores */
   UNI_nova_close(nd);
   kill(process_ID, SIGKILL);
/*     kill(getppid(), SIGINT);       Only the parent process now does this  */
   Exit(ownpname, " ");
}






/*(a+out) ------------------------------------------------- */
user_interface(semid)
int semid;
/*(b+out) -------------------------------------------------
Procedure Name
User_interface - interacts with the user for PAW cmnds.
Synopsis
Description
user_interface(semid) interacts with the user for PAW cmnds.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   char msg[80], string[102], pname2[20];
   int flag, index, msqid, j;
   char header[20], arg_string[20];
   char oldstring[HISTORY][102];
   int exec_flag = TRUE;
   int opt = 0, i;
   Table entry;
   struct sigvec vec;

   printf("Waiting for  ANALYZER to initialize...");
   Update_Sem_Wait(semid,-1); /* wait for analyzer to initialize */
   printf(" done.\n");
   Update_Sem_Wait(semid,1);/* enable things */
/* this code attempts to intercept control-C */
#ifdef SGI
   vec.sv_handler = (int (*)()) interrupt_handler;
#else
   vec.sv_handler = (void (*)()) interrupt_handler;
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(SIGINT, &vec, (struct sigvec *) NULL);
#else
 sigvec(SIGINT, &vec, (struct sigvec *) NULL);

#endif

   strcpy(pname2, ownpname); 
   /* strcpy(pname2, "analyzer"); */
   index = -1;
   j = 0;
   while ((index==-1) && (j<5) )  {
     if (Attach_Table() == -1) printf ("user: Error in Attach_Table()\n");
     index = Search_Table(pname2);
     if (index == -1) {
        printf("user: Error finding self (%s) in Search_Table()\n", pname2);
	if (j==4) exit (1);
	Detach_Table();
	sleep(2);
     }
     j++;
   }

   Read_from_Table(&entry, index);
   msqid = entry.msgq_id;
   if (debug_level >= 20) printf("user: msgq_id = %d\n", msqid);
 
   flag = TRUE;
   while (flag == TRUE) {
/**	Display_Sem(semid);*/
      Update_Sem_Wait(semid, -1);
         printf("PAW > ");
         gets(msg);
      Setval_Sem(semid,1);
/*      Update_Sem_Wait(semid, 1);*/
/*           handle the case where this is @command (ie, comproc command)  */
      if (strlen(msg) >= 2) {
	if (msg[0] == '@')  {
	  strcpy(string,"ComProc ");
	  strcat(string,&msg[1]);
	  strcpy(msg,string);
	}
      }
      strcpy(string, pname2);
      strcat(string, " ");
      strcat(string, msg);
/*           if it is not in the command dictionary, pass it to paw via pawc */
      if (strlen(msg) > 0) {
      
        /* display list of past commands*/
        if (msg[0] == '!' && strlen(msg) == 1) { /* print old cammands */
          printf("\n");
          for(i=0; i < opt ; i++) {
             if(strncmp(oldstring[i], "pawc", 4) == 0)  
                printf("\t%d\t%s\n",i,oldstring[i]+5);
             else 
                printf("\t%d\t%s\n",i,oldstring[i]);
          }
          exec_flag = FALSE;      
        }
        
        /* repeat last message  ie. !!*/
        else if (msg[0] == '!' && msg[1] == '!'&& strlen(msg) == 2) {
          strcpy(string, oldstring[opt-1]);
          if (opt == HISTORY) { copy_history(oldstring); opt--; }
          strcpy(oldstring[opt++], string);  
        }
        
        /* ! with a event # ie. !5 */
        else if (msg[0] == '!' && msg[1]>= '0' && msg[1]<= '9' ) {
           i = atoi(msg+1);
           if ( i < opt) {
              strcpy(string, oldstring[i]);
              if (opt == HISTORY) { copy_history(oldstring); opt--; }
              strcpy(oldstring[opt++], string);
           }
           else {
              printf("Enter a correct number \n");
              exec_flag = FALSE;
           }    
        }

        /* ! with a string also !hi for !hi/pl 2 */
        else if (msg[0] == '!') {
           i = opt;
           while ( i > -1 && strncmp(msg+1, oldstring[i]+5, strlen(msg+1)) != 0)
           {
              i--;
           };
           if (i == -1) { 
              printf("NO match for %s was found.\n",msg+1);
              exec_flag = FALSE;
           }
           else {
              strcpy(string, oldstring[i]);
              if (opt == HISTORY) { copy_history(oldstring); opt--; }
              strcpy(oldstring[opt++], string);
           }
        }
        else if ( !(strcasecmp(msg,"exit") == 0 || strcasecmp(msg,"quit") == 0 )) {
	      strcpy(string, "pawc ");
              strcat(string, msg);
              if (opt == HISTORY) { copy_history(oldstring); opt--; }

              strcpy(oldstring[opt++], string);
	  
        }
        else     { /* exit command */
	         strcpy(string,msg);  
                 if (opt == HISTORY) { copy_history(oldstring); opt--; }
	         strcpy(oldstring[opt++], string);
                Send_Message(ownpname, string);
		/* Delete_Sem(semid); */
		exit(0);

        }
        if (exec_flag == TRUE) {
          Send_Message(ownpname, string);
          sleep(2);
        }
        else {
           exec_flag = TRUE;
           sleep(2);
        }   
      }
   }
}




/*(a+out) ------------------------------------------------- */
void interrupt_handler()
/*(b+out) -------------------------------------------------
Procedure Name
interrupt_handler - 
Synopsis
Description
interrupt_handler 

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
  /*  printf("don_int_hanbler %d\n",getppid());
   kill(getppid() ,SIGINT);*/

   printf("\n *** Break ***\n");

   /* if (Delete_Sem(semid) == ERROR)
      if (debug_level >=10) printf("Error in Delete_Sem()\n"); 
   exit(1); */
}
/*(a+out) ------------------------------------------------- */
void special_handler()
/*(b+out) -------------------------------------------------
Procedure Name
special_handler - 
Synopsis
Description
interrupt_handler 

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
  /*  printf("don_int_hanbler %d\n",getppid());
   kill(getppid() ,SIGINT);*/
/* restart  */
	flush_buf();
/* ensure the semaphore is set ok */
	Display_Sem(semid);
	Setval_Sem(semid,1);
	Display_Sem(semid);
/* we will not return  from template main (I hope) */
	longjmp(env,1);
       Template_Main((int (*)()) Process_Event, NULL, 
                    (int (*)()) display_macro);
	ExitA(); /* exit and cleanup */
	exit(0);
   /* if (Delete_Sem(semid) == ERROR)
      if (debug_level >=10) printf("Error in Delete_Sem()\n"); 
   exit(1); */
}



/*(a+out) ------------------------------------------------- */
int display_macro()
/*(b+out) -------------------------------------------------
Procedure Name
display_macro - .
Synopsis
Description
display_macro .

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{   
   struct timeval tp;
   struct timezone tzp;
   char macro_cmd[80];

   gettimeofday(&tp, &tzp);
    
   if (macro.on == TRUE) {
      if (tp.tv_sec - macro.lastdisplay > macro.interval) {
         strcpy(macro_cmd, "exec ");
         strcat(macro_cmd, macro.macroname);
         strcat(macro_cmd, " ");
         Paw(ownpname, macro_cmd);
         macro.lastdisplay = tp.tv_sec;
      }
   }
   
}


#ifdef YESDYN        /*  Start of Dynamic link routines  LIST, DELETE_ALL,  */
                     /*     *SEARCHFILE, *SEARCHPROC                        */

/*(a+out) ------------------------------------------------- */
int List(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
List - gives a list of files link and they entry points
Synopsis
Description
List 

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   struct procedures *p;
   struct files *f;
 
   f = first_file;
   while (f != NULL) {
      if (debug_level >= 0) printf("file: %s\n",f->filename);
      p = f->first_proc;
      if (debug_level >= 0) printf("\tEntries:\n");
      while (p != NULL) {
         if (debug_level >= 0) printf("\t\t%s\n",p->procname);
         p = p->next;
      }
      f= f->next;
   }
   return(TRUE);
}



/*(a+out) ------------------------------------------------- */
void Dlt_allp(f)
struct files *f;
/*(b+out) -------------------------------------------------
Procedure Name
Dlt_allp - .
Synopsis
Description
Dlt_allp .

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
 struct procedures *p;
 
 while (f->first_proc != NULL) {
    p = f->first_proc;
    f->first_proc = p->next;
    free(p);
 }

}




/*---------------------------------------------------------*/
/* The next two function can not be made into man page     */
/*---------------------------------------------------------*/         

struct files *srchfile(filename, f)
char *filename;
struct files *f;
{
 while (f != NULL) {
    if (strcmp(filename,f->filename) == 0)
        return(f);
    else
        f = f->next;
 }
     
 return(f);  
}

struct procedures *srchprc(procname, p)
char *procname;
struct procedures *p;
{
 while (p != NULL) {
    if (strcmp(procname,p->procname) == 0)
        return(p);
     else
        p = p->next;
 }     
 return(p);   

}
#endif               /*  End of Dynamic link routines LIST, DELETE_ALL,  */
                     /*     *SEARCHFILE, *SEARCHPROC                     */


void copy_history(commandlist)
char commandlist[][102];
{
 int i;

 for(i = 0; i != HISTORY-1;i++) {
    strcpy(commandlist[i], commandlist[i+1]);
 }
 return;
}


send_message_(prcnam,string)
char *prcnam,*string;
{       Send_Message(prcnam,string);
        return;
}

/*------------------------------------------------------------*/
/* the same as Parse() in cmdlib.c but it also looks for "()" */
/* By donald brenner 7/5/93                                   */
/*------------------------------------------------------------*/
Parse2(string, first, rest)
char *string, *first, *rest;
{
   char temp[CMD_LENGTH];
   int  i, j;
 
   i = 0;
   while (((string[i]==')') | (string[i]=='(') | (string[i]==' ') | (string[i]=='\t') | (string[i]=='\0')) & (i < strlen(string)))
      i++;
   j = i;
   if (string[j] == '=') j++;
   else while (((string[j] != ')') & (string[j] != '(') & (string[j] != ' ') & (string[j] != '\t') &
                (string[j] != '=')) & (j < strlen(string))) j++;
   strncpy(first, string+i, j-i);
   first[j-i] = '\0';
   strcpy(rest, string+j);
}

